; ScopeClock.asm -- Main source for Dutchtronix Oscilloscope Clock
;
;  Copyright  2008 Jan P.M. de Rie
;
;  All Rights Reserved
;
;  This file is part of the Dutchtronix Oscilloscope Clock Distribution.
;  Use, modification, and re-distribution is permitted subject to the
;  terms in the file named "LICENSE.TXT", which contains the full text
;  of the legal notices and should always accompany this Distribution.
;
;  This software is provided "AS IS" with NO WARRANTY OF ANY KIND.
;
;  This notice (including the copyright and warranty disclaimer)
;  must be included in all copies or derivations of this software.
;
; Firmware for Dutchtronix AVR Oscilloscope Clock 
;
;	The Display tables are generated by a separate program.
;
;	03/10/2007: V2.0 completed
;	04/23/2007: V2.4 completed
;	06/27/2007: V2.6 completed
;	01/25/2008:	V3.0 completed
;	09/17/2008:	V3.3 completed
;	10/02/2008:	V3.5 completed
;
; The RTC (or an external source) interrupts (using INT0) once a second to update the time (PPS signal)
;	Disabled in V3.4 (CRT board version). 
;
;
; General design
;
; Main:
;	Initialize
;	loop forever calling (Menu) ClockWorks and checking for user input flags
;	Also process Modification of Time/Date.
;
; ClockWorks:
;	Refresh
;	Check various bits set by interrupt or other functions
;
; (Menu)Refresh:
;	loop scanning tables in code space with coordinate pairs to draw vectors
;	showing the clock, hands, numbers, status fields and menu
;
; User control through button switch and on-screen menu.
;		Serial RS-232 also available, fully buffered and interrupt driven.
;
;
; Register conventions
;
;	Y (r28,r29) is used for global variables access
;	r8, r9 used as Grid Offset values (global)
;	r14 and r15 used as literals (global)
;	r25 used as a status register (global)
;	r11, r12 used a X and Y offset in Clock/Menu Refresh
;	r24,r22 hold X0,Y0 coordinate in Clock/Menu Refresh function
;	r20,r18 hold X1,Y1 coordinates in Clock/Menu Refresh function
;	Z points to current coordinates table entry in Refresh function
;
;	functions usally preserve registers
;
; See scopeclockconfig.h for port definitions
;
;
;	date:							Apr-15-07		Jun-17-07		Jan-19-08		Sep-7-08
;refresh rate with full clock face	
;	 and numeric AM/PM:				150-160 Hz.		175-185	Hz		202-207 Hz		192-198
;refresh rate with minimal
;	 clock face:					260 Hz			400 Hz (??)		285-289 Hz		248 (12 hr clock face, no numeric)
;
;
#define Hardware20	0
;xx#define Hardware30 0
#define Hardware31 1
;
; Software V3.3 and up used for hardware 3.1 which uses 2 push button switches on PB4 and PB5.
; These pins are permanent input pins so outputting data to them as part of the DAC access
; is ignored. Pins cannot be set as interrupt since the DAC access code would trigger them.
; DAC access nibbles are reversed from Hardware20
; Uses AD7302, Normal Z control (High means Beam On)
;
#define MajorVersion 3
#define MinorVersion 5

#define Mega32 0
#define Mega168 0
#define Mega328p 1

#define ds1370 0
#define pcf8563 1
;xxx#define AD7302	!Hardware20

#define DEBUG 0&(Mega32 | Mega328p)
#define	DOTSPERCHARPIXEL 4
;xxx#define	PIXELDELAY 0

;xxx#define STATISTICS 1&DEBUG&SERIALIO
;xxx#define	TIMEDEBUG 0&DEBUG

;#define RS232HACK	Hardware20
;#define	SPACESAVINGHACK (Mega168 & !DEBUG)

#define	DRGcode		0
#define USESW2		1
#define	MULTIAPP	1
#define	DEMOMODE	1
#define SERIALIO	1
#define	DIAL24		1
#define FUNMODES	1
;xx#define DOUBLEFLASH	1
#define	GPS			1
;xxx#define BEAMPARK	0
#define	CHRONOMETER	1
#define EEPROMHELP	!(Mega32|Mega328p)
#define	ZINVERTED	Hardware20		;Tek 400 Family convention is inverted
;xxx#define	PATENTISSUE 0
;xxx#define POLLBUTTON	(!Hardware20)
;xxx#define NETFREQUENCY 0
;xxx#define CLOCKLEDENABLED
#define MULTIBAUD	1
#define AVRDUDEHACK	0			;change for new releases

;xxx#define SIGNATURE (Mega32 | Mega168 | Mega328p)

.nolist
#if Mega32
.include <m32def.inc>
#elif Mega328p
.include <m328pdef.inc>
#elif Mega168
.include <m168def.inc>
#endif
.list

.include "./avr.inc"
.include "./scopeclockconfig.h"
;
; Permanenty assigned registers
;
; the Y register is used for global variables access
;
.def	_0			= r15	;Zero register
.def	_1			= r14	;One register
.def	G_FLAGS 	= r25	;Global Flags
.def	GXOffset	= r8
.def	GYOffset	= r9

.include ".\constants.h"
.include ".\data.h"

.include ".\dac.h"
.include ".\rtc.h"

;----------------------------------------------------------;
; Program code area

#define	JMP jmp

#if DEBUG
#define	RCALL	call
#define	CALL	call
#else
#define	RCALL	rcall
#define	CALL	call
#endif

	.cseg
	JMP		Main				;0 Reset


#if DEBUG

#if Mega32
	JMP		INT0Handler			;1 External INT0
	JMP		INT1Handler			;2 External INT1
	JMP		BadVector3			;3 External INT2
	JMP		TC2MatchHandler		;4 Timer/Counter2 Compare Match
	JMP		BadVector5			;5 Timer/Counter2 Overflow
	JMP		BadVector6			;6 Timer/Counter1 Capture Event
	JMP		TC1MatchHandler		;7 Timer/Counter1 Compare Match A
	JMP		BadVector8			;8 Timer/Counter1 Compare Match B
	JMP		BadVector9			;9 TC1 overflow
	JMP		TC0MatchHandler		;10 TC0 compare Match
	JMP		BadVector11			;11 TC0 overflow
	JMP		BadVector12			;12 SPI Serial Transfer Complete
	JMP		SIG_USART_RECV		;13 USART Rx complete
	JMP		BadVector14			;14 USART Data Register empty
	JMP		SIG_USART_TRANS		;15 USART Tx complete
	JMP		BadVector16			;16 ADC conversion complete
	JMP		BadVector17			;17 EEProm Ready
	JMP		BadVector18			;18 Analog Comparator
	JMP		BadVector19			;19 TWI
	JMP		BadVector20			;20 SPM Ready
#elif (Mega168 | Mega328p)
	JMP		INT0Handler			;1 External INT0
	JMP		BadVector2			;2 External INT1
	JMP		BadVector3			;3 Pin Change Interrupt Request 0
	JMP		BadVector4			;4 Pin Change Interrupt Request 1
	JMP		PinChangeInterrupt	;5 Pin Change Interrupt Request 2
	JMP		BadVector6			;6 Watchdog Time-out Interrupt
	JMP		TC2MatchHandler		;7 Timer/Counter2 Compare Match A
	JMP		BadVector8			;8 Timer/Counter2 Compare Match A ??? probably B
	JMP		BadVector9			;9 Timer/Counter2 Overflow
	JMP		BadVector10			;10 TC1 capture
	JMP		TC1MatchHandler		;11 TC1 compare Match A
	JMP		BadVector12			;12 TC1 compare Match B
	JMP		BadVector13			;13 TC1 overflow
	JMP		TC0MatchHandler		;14 TC0 compare Match A
	JMP		BadVector15			;15 TC0 compare Match B
	JMP		BadVector16			;16 TC0 overflow
	JMP		BadVector17			;17 SPI Serial Transfer Complete
	JMP		SIG_USART_RECV		;18 USART Rx ready
	JMP		BadVector19			;19 USART Tx sfr empty
	JMP		SIG_USART_TRANS		;20 USART Tx ready
	JMP		BadVector21			;21 ADC conversion complete
	JMP		BadVector22			;22 EEProm Ready
	JMP		BadVector23			;23 Analog Comparator
	JMP		BadVector24			;24 TWI
	JMP		BadVector25			;25 SPM Ready
#endif
	

BadVector1:
	rjmp	BadVector1
BadVector2:
	rjmp	BadVector2
BadVector3:
	rjmp	BadVector3
BadVector4:
	rjmp	BadVector4
BadVector5:
	rjmp	BadVector5
BadVector6:
	rjmp	BadVector6
BadVector7:
	rjmp	BadVector7
BadVector8:
	rjmp	BadVector8
BadVector9:
	rjmp	BadVector9
BadVector10:
	rjmp	BadVector10
BadVector11:
	rjmp	BadVector11
BadVector12:
	rjmp	BadVector12
BadVector13:
	rjmp	BadVector13
BadVector14:
	rjmp	BadVector14
BadVector15:
	rjmp	BadVector15
BadVector16:
	rjmp	BadVector16
BadVector17:
	rjmp	BadVector17
BadVector18:
	rjmp	BadVector18
BadVector19:
	rjmp	BadVector19
BadVector20:
	rjmp	BadVector20
BadVector21:
	rjmp	BadVector21
BadVector22:
	rjmp	BadVector22
BadVector23:
	rjmp	BadVector23
BadVector24:
	rjmp	BadVector24
BadVector25:
	rjmp	BadVector25
#else

	.org	INT0addr
	JMP		INT0Handler			;1 External INT0

#if Mega32
	.org	INT1addr
	JMP		INT1handler			;2 External INT1
#endif

#if (Mega168 | Mega328p)
	.org	PCI2addr
	JMP		PinChangeInterrupt	;5 Pin Change Interrupt Request 2
#endif
	
#if Mega32
	.org	OC2addr				;TC2 Match
#elif (Mega168 | Mega328p)
	.org	OC2Aaddr			;TC2 Match A
#endif
	JMP		TC2MatchHandler

	.org	OC1Aaddr
	JMP		TC1MatchHandler		;9 TC1 Match A

#if Mega32
	.org	OC0addr				;TC0 Match
#elif (Mega168 | Mega328p)
	.org	OC0Aaddr			;TC0 Match A
#endif
	JMP		TC0MatchHandler		;16 TC0 Match

	.org	URXCaddr			;18 Serial In Received
	JMP		SIG_USART_RECV

	.org	UTXCaddr			;20 Transmit Complete
	JMP		SIG_USART_TRANS

#endif		// DEBUG

.include "./Flashtabs.asm"

;
; Initialize INT0
;
; TIMEMARK signal is Positive Going Pulse
;
InitINT0:
;
; configure the INT0 pin with a pull-up resistor
; to prevent random interrupts if unconnected
;
	sbi		PPS_IN				;enable pull-up
	CALL	InitRTCInt			;setup for Interrupt
#if Mega32
	in		r16,MCUCR
	ori		r16,((1<<ISC01)|(0 <<ISC00)) ;set ISC00 and ISC01 to INT0 on falling edge
	lds		r17,PPSTrigger				;different inits needed
	sbrc	r17,0
	ori		r16,((1<<ISC01)|(1 <<ISC00)) ;set ISC00 and ISC01 to INT0 on rising edge (GPS)
	out		MCUCR,r16
	in		r16,GICR
	ori		r16,(1<<INT0)		;enable INT0
	out		GICR,r16
#elif (Mega168 | Mega328p)
	ldi		r16,((1<<ISC01)|(0 <<ISC00)) ;set ISC00 and ISC01 to INT0 on falling edge
	lds		r17,PPSTrigger				;different inits needed
	sbrc	r17,0
	ldi		r16,((1<<ISC01)|(1 <<ISC00)) ;set ISC00 and ISC01 to INT0 on rising edge (GPS)
	sts		EICRA,r16
	sbi		EIMSK,INT0			;enable INT0
#endif
	ret

EnableINT0:
#if Mega32
	in		r16,GICR
	ori		r16,(1<<INT0)		;enable INT0
	out		GICR,r16
#elif (Mega168 | Mega328p)
	sbi		EIMSK,INT0			;enable INT0
#endif
	ret

DisableINT0:
#if Mega32
	in		r16,GICR
	andi	r16,~(1<<INT0)		;disable INT0
	out		GICR,r16
#elif (Mega168 | Mega328p)
	cbi		EIMSK,INT0			;disable INT0
#endif
	ret
;
; Handler for External Interrupt 0
;
; Triggered by RTC (or external source) once a second (1PPM)
; Sets flag to be acted upon in ClockWorks
;
INT0Handler:
	push	r16
	in		r16,SREG
	push	r16
	lds		r16,ClockIsRunning
	tst		r16
	breq	INT0HL100
	sbr		G_FLAGS,(1<<fUpdSecond)	;Tell ClockWorks we need to advance a second
	std		TimeTicks,_0			;start fractional seconds count
INT0HL100:
	pop		r16
	out		SREG,r16
	pop		r16
	reti
;
; TC0 is used to program the push button
;
; It tracks the duration of a down push and it does key debouncing
;
; Initialize TC0
#if (Mega168 | Mega328p)
; Set prescaler to clkio/1024. At 20 Mhz, this is 51.2 uSec
; Using a Timer0CountUp value of 127 yields an match every 6.502 mSec
#elif Mega32
; Set prescaler to clkio/1024. At 16 Mhz, this is 64 uSec
; Using a Timer0CountUp value of 102 yields an match every 6.528 mSec
#endif
;
InitTC0:
	push	r16
#if (Mega168 | Mega328p)
	out		TCCR0A,_0
	outi	TCCR0B,(1<<CS02)|(0<<CS01)|(1<<CS00)
	outi	OCR0A,Timer0CountUp
#elif Mega32
	outi	TCCR0,(1<<CS02)|(0<<CS01)|(1<<CS00)
	outi	OCR0,Timer0CountUp
#endif
	pop		r16
	ret
;
; Only called from the TC0 interrupt handler itself
;
StartTC0S1Countdown:
	ldi		r16,low(TIMER0TIMEOUT)
	std		Timer0S1Counter,r16
	ret

StopTC0S1Countdown:
	std		Timer0S1Counter,_0
	ret

StartTC0S2Countdown:
	ldi		r16,low(TIMER0TIMEOUT)
	sts		Timer0S2Counter,r16
	ret

StopTC0S2Countdown:
	sts		Timer0S2Counter,_0
	ret
;
; Enable TC0 timer for key debouncing
;
; USES r16
;
StartKeyDebouncing:
EnableTC0:
	sbr		G_FLAGS,(1<<fDebouncing)
	std		Timer0S1Counter,_0
#if USESW2
	sts		Timer0S2Counter,_0
#endif
#if DEBUG
	ldi		r16,150
	sts		TC0DebugCnt,r16
#endif
#if (Mega168 | Mega328p)
	outi	TIFR0,(1<<OCF0A)	;clear pending match
	stsi	TIMSK0,(1<<OCIE0A)	;Enable TC0.match
#elif Mega32
	outi	TIFR,(1<<OCF0)		;clear pending match
	outi	OCR0,Timer0CountUp
	in		r16,TIMSK
	ori		r16,(1<<OCIE0)		;Enable TC0.match
	out		TIMSK,r16
	RCALL	TurnOnLed2			;for key debugging
#endif
	ret
;
; Disable TC0 timer for key debouncing
;
; USES r16
;
DisableTC0:
#if (Mega168 | Mega328p)
	stsi	TIMSK0,(0<<OCIE0A)	;Disable TC0.match
#elif Mega32
	in		r16,TIMSK
	andi	r16,~(1<<OCIE0)		;Disable TC0.match
	out		TIMSK,r16
	RCALL	TurnOffLed2			;for key debugging
#endif
	ret
;
; Timer0 Match Handler.
;
TC0MatchHandler:
	push	r16
	out		TCNT0,_0
	sei		; TEST
	in		r16,SREG
	push	r16
	push	r17

;
; Decrement Timer0S1Counter iff != 0
;
	ldd		r16,Timer0S1Counter
	tst		r16
	breq	TC0L100				;skip test
	dec		r16					;subtract 1
	std		Timer0S1Counter,r16	;update value
	brne	TC0L100				;not zero yet
;
; the 8-bit Timer0S1Counter field is now 0
;
#if DEMOMODE
	lds		r16,DemoEnabled		;ignore if in demo mode
	tst		r16
	brne	TC0L100
#endif
	sbr		G_FLAGS,(1<<fTC0S1)	;mark TC0 timeout in flags
	std		S1Position,_0		;Pretend Button is up (ignore next fUPS1)
#if USESW2
	sts		fAutorepeat,_0
#endif
TC0L100:

#if USESW2
;
; Decrement Timer0S2Counter iff != 0
;
	lds		r16,Timer0S2Counter
	tst		r16
	breq	TC0L105				;skip test
	dec		r16					;subtract 1
	sts		Timer0S2Counter,r16	;update value
	brne	TC0L105				;not zero yet
;
; the 8-bit Timer0S2Counter field is now 0
;
	sbi		GPIOR0,fTC0S2		;mark TC0S2 timeout in flags
	sts		S2Position,_0		;Pretend Button is up (ignore next fUPS2)
TC0L105:
#endif

;
; bool_t DebounceSwitch2() Jack Ganssle embedded.com
;    static uint16_t State = 0; // Current debounce status
;    State=(State<<1) | !RawKeyPressed() | 0xe000;
;    if(State==0xf000)return TRUE;
;    return FALSE; 
; 
	ldd		r16,KeyStateS1
	lsl		r16					;shift left
	mov		r17,_1				;preset to UP	SPEED????
	sbis	SW_PIN,SW_BIT		;skip if HIGH (meaning UP)
	clr		r17					;set to DOWN
	or		r16,r17
	std		KeyStateS1,r16
	cpi		r16,0b10000000		;7 downs in a row
	brne	TC0L200
	ldd		r16,S1Position		;Is Button already DOWN (1)
	cp		r16,_1
	breq	TC0L300				;yes, ignore this event
	ldd		r16,G2_FLAGS
	sbr		r16,(1<<fDOWNS1)	;set DOWN in flags
	std		G2_FLAGS,r16
	std		S1Position,_1		;Button is down
	rcall	StartTC0S1Countdown
	rjmp	TC0L300
TC0L200:
	cpi		r16,0b01111111		;7 ups in a row
	breq	TC0L205				;yes, process
;
; This code is here to deal with spurious pushes that do not result in an fUP message
;
	cpi		r16,0b11111111		;button up for a while?
	breq	TC0L210
;
; KeyStateS1==0 (r16) indicates key has been down for at least 8 checks
;
#if USESW2
	tst		r16					;still down?
	brne	CMenuSA190			;no, out
	lds		r16,okToRepeat		;ok to auto repeat
	tst		r16
	breq	CMenuSA190
	sbrs	G_FLAGS,fTC0S1		;S1 TC0 flag (S1 long push)
	rjmp	CMenuSA190			;not set
	lds		r16,fAutorepeat		;already repeating ?
	tst		r16
	brne	CMenuSA97			;yes
	std		S1Position,_1		;UNDO Pretend Button is up (we want the next fUPS1) IS THIS STILL TRUE?
	ldi		r16,AUTOREPEATVAL
	sts		autorepeatcnt,r16
	sts		fAutorepeat,_1		;set flag
	rjmp	CMenuSA190			;out
CMenuSA97:
	lds		r16,autorepeatcnt	;current cnt
	dec		r16
	sts		autorepeatcnt,r16	;no flags affected
	brne	CMenuSA190			;still waiting. out
;
; reached 0. Insert an fUPS1 event.
;
	sbr		G_FLAGS,(1<<fUPS1)	;set UP in flags
	ldi		r16,AUTOREPEATVAL	;continue autorepeat
	sts		autorepeatcnt,r16
CMenuSA190:
#endif
	rjmp	TC0L300				;out
;
; 7 ups in a row
;
TC0L205:

#if USESW2
	lds		r16,fAutorepeat		;autorepeating?
	tst		r16
	breq	CMenuSA95			;no, short push
	std		S1Position,_0		;Button is up
	sts		fAutorepeat,_0		;cancel any autorepeat
	cbr		G_FLAGS,(1<<fTC0S1)
	ldd		r16,G2_FLAGS
	cbr		r16,(1<<fDOWNS1)	;clear DOWN in flags
	std		G2_FLAGS,r16
	rjmp	TC0L210				;stop debouncing
CMenuSA95:
#endif


	ldd		r16,S1Position		;Is Button already UP (0)
	tst		r16
	breq	TC0L300				;yes, ignore this event
	sbr		G_FLAGS,(1<<fUPS1)	;set UP in flags
	std		S1Position,_0		;Button is up
	ldd		r16,G2_FLAGS
	cbr		r16,(1<<fDOWNS1)	;clear DOWN in flags
	std		G2_FLAGS,r16
TC0L210:
	rcall	StopTC0S1Countdown
	rcall	StopKeyDebouncing	;stop debouncing if both S1 & S2 done
TC0L300:

#if USESW2
	lds		r16,KeyStateS2
	lsl		r16					;shift left
	mov		r17,_1				;preset to UP	SPEED????
	sbis	SW_PIN,SW2_BIT		;skip if HIGH (meaning UP)
	clr		r17					;set to DOWN
	or		r16,r17
	sts		KeyStateS2,r16
	cpi		r16,0b10000000		;7 downs in a row
	brne	TC0L400
	lds		r16,S2Position		;Is Button already DOWN (1)
	cp		r16,_1
	breq	TC0L500				;yes, ignore this event
	ldd		r16,G2_FLAGS
	sbr		r16,(1<<fDOWNS2)	;set DOWN in flags
	std		G2_FLAGS,r16
	sts		S2Position,_1		;Button is down
	rcall	StartTC0S2Countdown
	rjmp	TC0L500
TC0L400:
	cpi		r16,0b01111111		;7 ups in a row
	breq	TC0L405				;yes, process
;
; This test is here to deal with spurious pushes that do not result in an fUP message
;
	cpi		r16,0b11111111		;button up for a while?
	breq	TC0L410
	rjmp	TC0L500
TC0L405:
	lds		r16,S2Position		;Is Button already UP (0)
	tst		r16
	breq	TC0L500				;yes, ignore this event
	sbi		GPIOR0,fUPS2		;set UP in flags
	sts		S2Position,_0		;Button is up
	ldd		r16,G2_FLAGS
	cbr		r16,(1<<fDOWNS2)	;clear DOWN in flags
	std		G2_FLAGS,r16

TC0L410:
	rcall	StopTC0S2Countdown
	rcall	StopKeyDebouncing	;stop debouncing if both S1 & S2 done

TC0L500:
#endif
	pop		r17
	pop		r16
	out		SREG,r16
	pop		r16
	reti
;
; always called with Interrupts OFF
;
; Stop debouncing if S1 and S2 have both been up for a while
;
StopKeyDebouncing:
#if USESW2
	ldd		r17,KeyStateS1		;all ones means UP
	lds		r16,KeyStateS2
	and		r17,r16
	cpi		r17,-1				;done if still all ones
	brne	StopKDL100
#endif
	cbr		G_FLAGS,(1<<fDebouncing)
	rcall	DisableTC0			;stop key debouncing timer
StopKDL100:
	ret

PinChangeInterrupt:
	rjmp	PC

;
; TC1 is the general purpose timer
;
; Used for 1 PPS timeout checking, field flashing timing
;
; We use the 16-bit timer for this purpose since it will
; get us closest to precisely 1 second.
;
; Initialize TC1
; Set prescaler to clkio/256. At 20 Mhz, this is 12.8 uSec 
;
InitTC1:
#if Mega32
	outi	TCCR1B, 0b00000100	;TC1.ck = 16Mhz/256 = 16 uSec
	outi	OCR1AH,Timer1CountUpH
	outi	OCR1AL,Timer1CountUpL
#elif (Mega168 | Mega328p)
	stsi	TCCR1B, 0b00000100	;TC1.ck = 20Mhz/256 = 12.8 uSec
	stsi	OCR1AH, Timer1CountUpH
	stsi	OCR1AL, Timer1CountUpL
#endif
	rcall	EnableTC1
	ret
;
; TC1 countdown used to timeout CMOD/CMENU states
; an interrupt could occur while doing the stdw (2 instruction)
;
StartTC1Countdown:
	ldiw	A,TIMER1TIMEOUT
StartTC1CL100:
	cli
	stdw	Timer1Counter,A
	sei
	ret
;
; TC1 countdown.
;	Used to wait 4 seconds in demo mode
;	Also used for GPS Processing
;
StartTC1Countdown4Sec:
	ldiw	A,TIMER1TIMEOUT4SEC
	rjmp	StartTC1CL100
;
; return 0 in r16 if counter is NULL, else return 1 in r16
;
TestTC1Countdown:
	push	r17
	ldd		r16,Timer1Counter
	ldd		r17,Timer1Counter+1
	or		r16,r17
	breq	TestTC1CDL100		;0 result, done
	mov		r16,_1
TestTC1CDL100:
	pop		r17
	ret

;
EnableTC1:
#if Mega32
	in		r16,TIMSK
	ori		r16,(1<<OCIE1A)		;Enable TC1.match
	out		TIMSK,r16
#elif (Mega168 | Mega328p)
	stsi	TIMSK1, (1<<OCIE1A)	;Enable TC1.match
#endif
	std		Timer1Counter,_0
	std		Timer1Counter+1,_0
	ret
;
DisableTC1:
#if Mega32
	in		r16,TIMSK
	andi	r16,~(1<<OCIE1A)	;Disable TC1.match
	out		TIMSK,r16
#elif (Mega168 | Mega328p)
	stsi	TIMSK1, (0<<OCIE1A)	;Disable TC1.match
#endif
	ret
;
; Timer1 Compare Match Handler.
;
; Operates at 60 Hz (16.666 mSecs)
;
; Use the 16 bit counter since it approaches 1 second closest.
;
; Maintains a timer in case INT0 fails; also used as a flash counter
;
; Other timers to be added (1 byte counter yields max 4.2 seconds delay)
;
TC1MatchHandler:
	push	r16
#if Mega32
	out		TCNT1H,_0
	out		TCNT1L,_0
#elif (Mega168 | Mega328p)
	sts		TCNT1H,_0
	sts		TCNT1L,_0
#endif
	sei
	in		r16,SREG
	pushw	A
	pushw	X
;
; Check if the push button switch S1 is closed.
; Skip if we are in debouncing mode
; if HIGH (set), button is UP. If LOW (clear), button is DOWN
;
	sbrc	G_FLAGS,fDebouncing		;skip test if already debouncing
	rjmp	TC1MHL200
#if USESW2
	in		r16,SW_PIN				;check if any button switch pin
	andi	r16,((1<<SW_BIT)|(1<<SW2_BIT)) ; is low (meaning down)
	cpi		r16,((1<<SW_BIT)|(1<<SW2_BIT))
	breq	TC1MHL200
	rcall	StartKeyDebouncing
#else
	sbis	SW_PIN,SW_BIT			;skip if HIGH (meaning UP)
	rcall	StartKeyDebouncing
#endif
TC1MHL200:
;
; Check if the push button switch S2 is closed.
; Reboot if it is
;
#if USESW2
	sbis	GPIOR0,fTC0S2			;did SW2 timeout
	rjmp	TC1MHL300				;no
	cbi		GPIOR0,fTC0S2			;clear
	CALL	InitiateSysReset
#else
	sbis	SW_PIN,SW2_BIT			;skip if HIGH (meaning UP)
	CALL	InitiateSysReset
#endif
TC1MHL300:
;
; Decrement Timer1Counter iff != 0
;
	lddw	X,Timer1Counter
	mov		r16,XL
	or		r16,XH
	breq	TC1L010					;skip test
	sbiw	Xl,1					;subtract 1
	stdw	Timer1Counter,X			;update value
	brne	TC1L010					;not zero yet
;
; the 16-bit Timer1Counter field is now 0
;
	sts		GPSOKToUpdate,_1		;ok to process GPS record
TC1L010:
;
;  countdown in case INT0 fails
;
	ldd		r16,PPSTimeOutCnt
	tst		r16						;timer active?
	breq	TC1L110					;no
	dec		r16
	brne	TC1L100
	sbr		G_FLAGS,(1<<fTimeOut)	;set TimeOut Flag
	ldd		r16,ReloadPPSTimeOutCnt ;reinitialize tick counter for timeout
TC1L100:
	std		PPSTimeOutCnt,r16
TC1L110:
;
; Countdown for flashing fields
;
	ldd		r16,FlashCount
	dec		r16
	brne	TC1L150
	sbr		G_FLAGS,(1<<fFLASH)		;set Flash-Bit
	ldi		r16,FLASHTICKS			;reload
TC1L150:
	std		FlashCount,r16

TC1HandlerOut:
	popw	X
	popw	A
	out		SREG,r16
	pop		r16
	reti

#if CHRONOMETER | DEBUG
;
; TC2 is used to run a 200Hz clock
;
; Initialize and Enable TC2
#if (Mega168 | Mega328p)
; Set prescaler to clkio/1024. At 20 Mhz, this is 51.2 uSec
; Using a Timer0CountDown value of 97 yields an overflow every 4.99 mSec (about 200 Hz)
#elif Mega32
; Set prescaler to clkio/1024. At 16 Mhz, this is 64 uSec
; Using a Timer0CountDown value of 78 yields an overflow every 4.99 mSec (about 200 Hz)
#endif
;
InitTC2:
	push	r16
#if (Mega168 | Mega328p)
	sts		TCCR2A,_0
	stsi	TCCR2B,(1<<CS22)|(1<<CS21)|(1<<CS20)
	stsi	OCR2A,Timer2CountUp
#elif Mega32
	outi	TCCR2,(1<<CS22)|(1<<CS21)|(1<<CS20)
	outi	OCR2,Timer2CountUp
#endif
	rcall	EnableTC2
	pop		r16
	ret

EnableTC2:
#if (Mega168 | Mega328p)
	outi	TIFR2,(1<<OCF2A)	;clear pending match
	stsi	TIMSK2,(1<<OCIE2A)	;Enable OC2.ov
#elif Mega32
	outi	TIFR,(1<<OCF2)		;clear pending match
	in		r16,TIMSK
	ori		r16,(1<<OCIE2)		;Enable OC2.ov
	out		TIMSK,r16
#endif
	ret
DisableTC2:
#if (Mega168 | Mega328p)
	stsi	TIMSK2,(0<<OCIE2A)	;Disable TC2.ov
#elif Mega32
	in		r16,TIMSK
	andi	r16,~(1<<OCIE2)	;Disable OC2.ov
	out		TIMSK,r16
#endif
	ret
;
; TC2 is an 8-bit counter
;
TC2MatchHandler:
	push	r16
	in		r16,SREG
	push	r16
#if Mega32
	out		TCNT2,_0
#elif (Mega168 | Mega328p)
	sts		TCNT2,_0
#endif
	lds		r16,ClockIsRunning
	tst		r16
	breq	TC2MHL10
	ldd		r16,TimeTicks
	inc		r16
	std		TimeTicks,r16
#if MULTIAPP
	lds		r16,Time2Ticks
	tst		r16
	breq	TC2MHL10
	dec		r16
	sts		Time2Ticks,r16
#endif
TC2MHL10:
	pop		r16
	out		SREG,r16
	pop		r16
	reti
#else
TC2MatchHandler:
	ret
#endif
;
Main:
;	cli

#if (Mega168 | Mega328p)
	CALL	WDTimerOff				;if we got here from the bootloader
;
; SP set by hardware
;
#else
	outi	SPL,low(RAMEND)			;initialize SP
	outi	SPH,high(RAMEND)
#endif

#if DEBUG
	ser		r18
#else
	clr		r18
#endif
	ldiw	Z,SRAM_START			;Preset RAM
	ldi		r17,SRAM_SIZE/256		;256 Bytes blocks
MainL1:
	clr		r16						;start at 256
MainL5:
	st		Z+,r18
	dec		r16
	brne	MainL5
	dec		r17
	brne	MainL1

;
; Initialize Permanently Assigned Registers
;
	ldiw	Y,GlobalVars			;permanently assigned to base of Global Vars
	clr		_0						;permanently assigned 0
	clr		_1						;permanently assigned 1
	inc		_1
	clr		G_FLAGS
	out		GPIOR0,_0
	clr		GXOffset
	clr		GYOffset

	ser		r16
	std		KeyStateS1,r16			;this prevents an initial UP
#if USESW2
	sts		KeyStateS2,r16			;this prevents an initial UP
#endif
	std		BatteryLow,r16			;special value!
	ldi		r16,CMODIDLE
	std		CModState,r16
	stdi	CMenuState,CMENUIDLE
;
; These are regular initializations but unneeded in RELEASE mode
; since all memory is set to 0 at startup.
;
#if DEBUG
	sts		ExtDbg1,_0
	sts		ExtDbg2,_0
	sts		SecondCnt,_0
	sts		RestartCounter,_0
	sts		DBGStage,_0
	sts		ExtraTimer,_0
#if CHRONOMETER
	clrsw	RefreshCnt
#endif
	std		G2_FLAGS,_0
	sts		RTCNotPresent,_0
#if DEMOMODE
	sts		DemoEnabled,_0
#endif
	std		ShowCalibration,_0
	std		DisplayRedrawFlg,_0
	std		S1Position,_0			;S1 is UP
#if USESW2
	sts		S2Position,_0			;S2 is UP
	sts		fAutorepeat,_0
	sts		okToRepeat,_0
#endif
	clrdw	PPSFlashPtr
	clrsw	BatFlashPtr				;unused
	clrdw	DialFlashPtr
	clrdw	CModFlashPtr1
	clrdw	CModFlashPtr2
	clrdw	CMenuFlashPtr
	std		RotationIndex,_0		;preset RotationIndex
	sts		SavedRotationIndex,_0	;preset SavedRotationIndex
	std		TextFlashField,_0
	clrdw	CTextFlashPtr
	sts		UsrNameDflt,_0
	std		TimeTicks,_0
	sts		Time2Ticks,_0
	sts		CurrNumericHPos,_0
	sts		SavedBeamParkVal,_0
	sts		GPSLastHrFlag,_0
	sts		ClockIsRunning,_0
;
; Preset ScanTbl with default data
;
	ldi		r18,(SCANTABLEN+SCANSTRTABLEN+SCANSSTRTABLEN+1)*4
	ldiw	X,ScanTbl
MainL22:
	st		X+,_0
	dec		r18
	brne	MainL22

#if GPS
;
; Initialize GPS related data
;
	sts		GPSInEnabled,_0			;Default Off
	std		UTCHrs,_0
	std		UTCMins,_0
	std		UTCSecs,_0
	std		UTCDay,_0
	std		UTCMonth,_0
	std		UTCYear,_0
	sts		GPSOffset,_0
	sts		GPSInCnt,_0
;
; True debug code
;
; clear buffer
;
	ldiw	X,GPSInBuf				;compute buffer ptr
	ldi		r16,GPSInBufLength
MainL25:
	st		X+,_0
	dec		r16
	brne	MainL25
	ldiw	X,GPSInBufGuard
	ser		r18
	st		X+,r18
	st		X+,r18
	st		X+,r18
	st		X+,r18
#endif		; GPS

; clear UsrName buffer
;
	ldiw	X,UsrNameBuf			;compute buffer ptr
	ldi		r16,MAXUSRNAMELEN+1
MainL30:
	st		X+,_0
	dec		r16
	brne	MainL30
	ldiw	X,UsrNameBufGuard
	ser		r18
	st		X+,r18
	st		X+,r18
	st		X+,r18
	st		X+,r18
#endif		; DEBUG

	ldi		r18,SCANTABLEN
	std		CurScanTblLen,r18
	ldiw	Z,DialData*2			;Dial Data
	stsw	ScanTbl_Dial,Z
	stdi	CurrPlayMode,FUNNORM
;
; initialize MScanTbl in SRam from Image in Flash
;
	ldiw	X,MScanTbl				;SRam
	ldiw	Z,MTabImage*2			;Flash based
	ldi		r17,MTabImageEnd-MTabImage
	lsl		r17						;times 2
MainL35:
	lpm		r16,Z+
	st		X+,r16
	dec		r17
	brne	MainL35

	sts		GPSOKToUpdate,_1		;time to process GPS record

 	ldi		r16,TIMEOUTTICKS		;initialize tick counter for timeout
	std		PPSTimeOutCnt,r16
	std		ReloadPPSTimeOutCnt,r16
	ldi		r16,FLASHTICKS			;initialize tick counter for flashing
	std		FlashCount,r16
	ldi		r16,BURNINCOUNTDOWN		;initialize countdown
	std		RotationCountdown,r16

	ldiw	Z,SpecialMainClockWorks	;default func ptr
	stdw	fpSpecialClockWorks,Z
;
; AVR initialization
;
#if (Mega168 | Mega328p)
;
; Disable unused parts of the AVR to reduce power consumption
;
; unusedL TWI, ADC
;
	ldi		r16,(1<<PRTWI)|(1<<PRADC)
	sts		PRR,r16
#endif
	rcall	InitTC0					;does NOT run until enabled
	rcall	InitTC1					;runs immediately
#if CHRONOMETER | DEBUG
	rcall	InitTC2
;	rcall	EnableTC2
#endif
	rcall	DACinit
	CALL	I2CInit

	CALL	InitLed
#if DEBUG
	RCALL	BlinkLed
#endif
	CALL	InitSwitch
	CALL	InitFlashTbl
;
; Read Configuration Data from EEProm
;
	CALL	ReadEEConfiguration
	rcall	ValidateNumericDisplay
	rcall	ValidateDialDisplay
;
; BUG discovered 11/4/2008
;
;	r16 is input argument but not set
;
	CALL	SetRotation				;
	CALL	ReadEEUsrName			;read user name from EEprom
	CALL	UART_Init				;default baudrate = 19,200
#if MULTIBAUD
	lds		r16,BaudVal				;override with value from EEprom
	CALL	SetBaudVal
#else
	lds		r0,GPSInEnabled			;After ReadEEConfiguration
	tst		r0						;modify if GPS input on
	breq	MainL40
	CALL	UART_Set4800
MainL40:
#endif
#if MULTIAPP
	ldi		r16,APPCLOCK			;start with clock app
	sts		CurrAppMode,r16
#endif

;
; See if the RTC was ever initialized. This requires that the EEPROM is
; not erased when programming the flash (Set Proper Fuse).
; Test first if an RTC is present. If not, skip this
; otherwise the EEPROM will be set when programming the AVR,
; even in parallel programming mode.
;
	CALL	StopRTCClockOut			;dummy call to test RTC presence
	lds		r0,RTCNotPresent
	tst		r0
	brne	MainL50
	lds		r0,ClockEverRun			;was the clock run before
	tst		r0
	brne	MainL50					;yes continue
	CALL	CleanInitRTC
MainL50:
	CALL	RTCReadTime
	CALL	RTCReadDate
	rcall	SetNumericDisplay
	rcall	SetDialDisplay
	rcall	UpdateDisplay
;
; If RTC not present, also skip the 1PPS signal synchronization.
;
	lds		r0,RTCNotPresent
	tst		r0
	brne	MainL220
;
; Synchronize to the next second boundary before starting the 1 PPS signal
;
	ldd		r17,AvrSecs
MainL210:
	push	r17
	rcall	ClockWorks				;interrupts still off
	CALL	RTCReadSecs
	pop		r17
	ldd		r16,AvrSecs
	cp		r16,r17
	breq	MainL210
	dec		r16						;UpdateSecs will increment again
	std		AvrSecs,r16
	rcall	UpdateSecs				;we just ran a second off the clock
#if MULTIAPP
;
; Initialize the Terminal Application
;
	CALL	InitTerm

	lds		r16,StartApp			;from EEprom
	cpi		r16,(APPGEN-APPCLOCK)	;validate
	brlo	MainL045				;ok
	ldi		r16,(APPCLOCK-APPCLOCK)	;reset to default
	CALL	UpdateStartApp_EEPprom	;r16 argument
MainL045:
	subi	r16,-APPCLOCK			;rebase
	sts		CurrAppMode,r16
	rcall	SetNumericDisplay		;again to reflect the actual app
#endif
;
; Initialize and Start the Clock
;
	rcall	InitINT0
	sts		ClockIsRunning,_1

MainL220:
	sei
;
; check if the clock started up in a DST window (RTC doesn't know about DST)
;
	rcall	DSTRetroApply			;needs interrupts for message
;
; BUG in V3.0. Need to redraw display if time was changed
;
	rcall	UPDS_DispAdr
;
; End V3.0 Bug Fix
;
#if SERIALIO
	CALL	UsrPrintNewLine
	CALL	UsrShowAll
	CALL	UsrPrintNewLine
#endif
;
; Select application to run. Only clock or terminal from boot.
;
#if MULTIAPP
	lds		r16,CurrAppMode
	cpi		r16,APPTERM
	brne	MainL400				;no, goto clock
	rcall	SetNumericDisplay
	jmp		RunTerm
MainL400:
#endif
;
; Main Program Forever Loop.
; No registers need to be saved by the called functions
;
MainLoop:
	rcall	ClockWorks				;show the clock and process events
	cbi		GPIOR0,fUPS2			;clear any pending S2 up action (no semantics)
;
; We are in IDLE state
; Did we receive an S1 Button Up Message (Menu mode)?
;
	sbrs	G_FLAGS,fUPS1
	rjmp	MainL500				;not UP
	cbr		G_FLAGS,(1<<fUPS1)


	rcall	MainMenuProcessing		;display and process the menu
#if MULTIAPP
;
; Did we change application in the menu? We are running Clock.
;
	lds		r16,CurrAppMode
	cpi		r16,APPTERM
	brne	CMSAVL100
	subi	r16,APPCLOCK			;rebase to 0
	CALL	UpdateStartApp_EEPprom	;r16 argument
	rcall	SetNumericDisplay
;	rcall	SetChronoDispBuf		;sb SetNumericDisplay
	JMP		RunTerm
CMSAVL100:
	cpi		r16,APPGEN
	brne	CMSAVL200
	JMP		RunFuncGen
CMSAVL200:
	cpi		r16,APPBOOT
	brne	CMSAVL300
	CALL	InitiateSysReset		;will reload CurrAppMode properly
CMSAVL300:
#endif
;
; We are in CMODIDLE state
; Did we receive a Button TimeOut Message (Change Time/Date)?
;
MainL500:
	sbrs	G_FLAGS,fTC0S1			;TC0 flag
	rjmp	MainL900
	cbr		G_FLAGS,(1<<fTC0S1)
;
; Turn off calibration before the clock goes in Time/Date Set Mode
;
	std		ShowCalibration,_0
	CALL	CalibrationOff
;
	lds		r16,CurrNumVal		;Date being displayed?
	cpi		r16,NUMDATE
	brne	MainL510			;no
;
; Enter Date Set Mode. Flash current date
; Don't stop clock when modifying the date, but stop updating the Numeric field
; while in Date Set Mode.
;
	ldi		r16,CMODPREYEAR|NOPUSH	;start at YEAR Field
	rjmp	MainL600
;
; Time Change Modes
;
MainL510:
	sts		ClockIsRunning,_0	;stop the clock
	CALL	DisablePPSTimeOut	;prevent timeout
;
; Enter Time Set Mode. Flash number, hands or leds depending on current state
;
	lds		r16,CurrNumVal		;if showing numbers, flash numbers
	cpi		r16,NUMOFF
	breq	MainL540
#if DEBUG
	cpi		r16,NUMDATE
	brne	MainL530
	rjmp	PC					;ERROR date update elsewhere
#endif
MainL530:
	ldi		r16,CMODPREHRNR|NOPUSH	;start at numeric hour mode
	rjmp	MainL600
;
; No Numeric Field. Flash hands/leds
;
MainL540:
	lds		r16,CurrDialVal		;Dial mode: hands or binary
	cpi		r16,DIALBIN
	brne	MainL550			;not binary
	ldi		r16,CMODPREHRBIN|NOPUSH	;start at binary hour mode
	rjmp	MainL600
MainL550:
	ldi		r16,CMODPREHRHAND|NOPUSH ;start at hour hands mode
MainL600:
	CALL	NextCModState
	rcall	StartTC1Countdown
#if USESW2
	sts		okToRepeat,_1			;ok to auto repeat
#endif
MainCModLoop:
	rcall	ClockWorks			;show the clock and process events
#if GPS
	rcall	DiscardGPSInput
#endif
	rcall	CModStateAdvance	;Time Change State Machine
	ldd		r16,CModState		;back to IDLE?
	cpi		r16,CMODIDLE
	breq	MainL700			;yes, exit
	rcall	TestTC1Countdown
	tst		r16
	brne	MainCModLoop			;still going
;
; CModState has timed out
;
	CALL	StopStateFlash2
	RCALL	StopStateTextFlash
	RCALL	CTimeCancel			;hack call to cleanup
	RCALL	CDateCancel			;cleanup
;
; done with CMOD states
;
MainL700:
;
; skip Clock Restart if we were setting the date
;
	lds		r16,CurrNumVal		;Date being displayed?
	cpi		r16,NUMDATE
	breq	MainL800
;	rcall	EnableTC2			;restart the chronometer
; TEST LINE BELOW
;	RCALL	StartRTCInt			;DOES NOT WORK WITH EXTERNAL PPS SIGNAL
	sts		ClockIsRunning,_1	;restart the clock
	CALL	EnablePPSTimeOut
MainL800:
	sts		GPSOKToUpdate,_1	;ok to process GPS record again
#if USESW2
	sts		okToRepeat,_0		;stop auto repeat
#endif
#if SERIALIO
	mov		r16,_1				;CR/LF
	CALL	DisplayAVRDateTime		;display on Serial Port
#endif

MainL900:
#if GPS
	lds		r0,GPSInEnabled		;are we processing GPS info
	tst		r0
	breq	MainL950			;no, command line input only
	rcall	CheckGPSInput		;process GPS records, if any
	rjmp	MainLoop
MainL950:
#endif

#if SERIALIO
;
; Was a char received on the UART
;
	sbrc	G_FLAGS,fRcvdChar	;skip Command Line Input processing if clear
;
; UART char received. Process command line input
;
	CALL	UsrCommand			;SpecialClockWorks will be called from here!
#endif			;SERIALIO

	rjmp	MainLoop
;
; end of Main Program Loop
;
;
; Main Program Loop for Menu Processing
;
; Separated since can also be called from Terminal Application
;
MainMenuProcessing:
;
; switch to menu mode. Start flashing the first entry
;
#if 1								;used in SERIALIO and DemoWait
	ldiw	Z,SpecialMenuClockWorks	;func ptr in Menu Mode
	stdw	fpSpecialClockWorks,Z
#endif
	CALL	CMenuSetTable			;populate Menu table with current values
	ldi		r16,CMENUNUM			;first state
	CALL	NextCMenuState
	ldiw	X,MScanTbl_NumVal		;First field to flash
	CALL	StartMenuStateFlash
	rcall	StartTC1Countdown		;start timeout timer
#if USESW2
	sts		okToRepeat,_1			;ok to auto repeat
#endif
MainCMenuLoop:
	rcall	MenuClockWorks			;show the menu and process events
#if GPS
	rcall	DiscardGPSInput			;could call CheckGPSInput, if desired
#endif
	RCALL	CMenuStateAdvance		;Menu Change State Machine
	ldd		r16,CMenuState			;back to IDLE?
	cpi		r16,CMENUIDLE
	breq	MainL475				;yes, exit
	rcall	TestTC1Countdown
	tst		r16
	brne	MainCMenuLoop			;still going
;
; Timing Out a CMENU state. Clean up any flash ptrs
;
	CALL	TimeOutEditUsrName		;only when in User Name Edit mode
	ldi		r16,CMENUIDLE			;force back to CMENUIDLE
	CALL	NextCMenuState
MainL475:
#if 1								;used in SERIALIO and DemoWait
	ldiw	Z,SpecialMainClockWorks	;restore default func ptr
	stdw	fpSpecialClockWorks,Z
#endif
	stdi	CMenuState,CMENUIDLE	;no menu state
	sts		GPSOKToUpdate,_1		;ok to process GPS record again
#if USESW2
	sts		okToRepeat,_0			;stop auto repeat
#endif
	ret
;	
; Set a funcptr with the appropriate function to call (clock or menu)
; This way, we can do I/O while in the menu screen
;
SpecialClockWorks:
	pushw	Z
	lddw	Z,fpSpecialClockWorks
	icall
	popw	Z
	ret
;
; Special version of ClockWorks, called from busy wait loops
; where registers need to be preserved
;
;
SpecialMainClockWorks:
	push	r0					;coordinates
	push	r1
	push	r16					;temp register
	push	r17					;DispTab index
	push	r18					;coordinate registers
	push	r19					;Used by DrawVector
	push	r20
	push	r22
	push	r24
	pushw	X
	pushw	Z
;
; these registers are used by the DrawVector function
;
	push	r21
	push	r23
	rcall	ClockWorks
	pop		r23					;Used by DrawVector
	pop		r21
	popw	Z
	popw	X
	pop		r24
	pop		r22
	pop		r20
	pop		r19
	pop		r18
	pop		r17
	pop		r16
	pop		r1
	pop		r0
	ret

;
; Workhorse function of the Scope Clock
; Does not need to preserve registers
; 	
ClockWorks:
#if CHRONOMETER
	lds		r0,ChronoEnabled
	tst		r0
	breq	ClockWL100
	lds		r16,RefreshCnt
	andi	r16,0b00000111		;every 8 refresh cycles
	brne	ClockWL100
	rcall	UpdateChronoBuf		;not always
ClockWL100:
#endif
	rcall	ClockRefresh		;update the display. No need to preserve registers
	rcall	CommonClockWorks
	ret
;
; Menu Mode version of ClockWorks
;
MenuClockWorks:
	rcall	MenuRefresh			;show the menu
	rcall	CommonClockWorks	;process events
	ret

;
; Special version of MenuClockWorks, called from busy wait loops
; where registers need to be preserved
;
SpecialMenuClockWorks:
	push	r0					;coordinates
	push	r1
	push	r16					;temp register
	push	r17					;DispTab index
	push	r18					;coordinate registers
	push	r19					;Used by DrawVector
	push	r20
	push	r22
	push	r24
	pushw	X
	pushw	Z
;
; these registers are used by the DrawVector function
;
	push	r21
	push	r23
	rcall	MenuClockWorks
	pop		r23					;Used by DrawVector
	pop		r21
	popw	Z
	popw	X
	pop		r24
	pop		r22
	pop		r20
	pop		r19
	pop		r18
	pop		r17
	pop		r16
	pop		r1
	pop		r0
	ret
;
; Code common between the Clock and Menu mode of operation
;
CommonClockWorks:
	sbrs	G_FLAGS,fUpdSecond	;Do we need to advance a second
	rjmp	CommonCWL100		;no
;
; code executed once a second
;
#if CHRONOMETER
;
; The fUpdSecond code is executed AFTER the 1PPS signal interrupt occured
; since we need to finish the Screen Refresh. Experimentation (on Atmega32)
; shows that we are at most 10 mSecs delayed (in clock mode)
;
#if DEBUG
	ldd		r16,TimeTicks
	cpi		r16,2
	brlt	CommonCWL041
	nop
CommonCWL041:
#endif
;	std		TimeTicks,_0		;re-initialize 100Hz counter
#endif

;
; reload PPSTimeOutCnt
;
	ldd		r17,PPSTimeOutCnt
	CALL	EnablePPSTimeOut	;reloads PPSTimeOutCnt
	sub		r16,r17
	breq	CommonCWL10			;same, PPSTimeOutCnt was reloaded
	cpi		r16,30				;less than 30 ticks passed?
	brsh	CommonCWL10			;no
;
; we have a very short second (<30 ticks)
;
	rjmp	CommonCWL080			;skip UpdateSecs

#if FUNMODES
CommonCWL10:
	ldd		r0,CurrPlayMode
	sbrc	r0,RUNCLOCKREVERSE
	rjmp	CommonCWL075
	rcall	UpdateSecs
	rjmp	CommonCWL080
CommonCWL075:
	rcall	ReverseSecs
CommonCWL080:
#else
	rcall	UpdateSecs
#endif		;FUNMODES


#if DEMOMODE
	lds		r17,DemoEnabled
	sbrs	r17,fDemoGoing
	rjmp	CommonCWL090
;
; if doing PPS warning demo, don't turn off PPSMissing
;
	sbrs	r17,fDemoPPS
	CALL	PPSMissingOff		;Turn off PPS missing warning (if on)
;
; if doing Burn-In Demo, move grid once a second
;
	sbrc	r17,fDemoBurnIn
	CALL	RotateXYOffset
	rjmp	CommonCWL095				;done
CommonCWL090:
#endif
	lddw	X,PPSFlashPtr		;FlashTbl entry
	mov		r0,XL				;test ptr
	or		r0,XH
	breq	CommonCWL095				;NULL, skip
	CALL	PPSMissingOff		;Turn off PPS missing warning
CommonCWL095:
	cbr		G_FLAGS,(1<<fUpdSecond)	;clear Seconds flag
;
; Add more items due at the seconds boundary here
;
; See if the 1 PPS signal is missing
;
CommonCWL100:
	sbrs	G_FLAGS,fTimeOut	;Did we get a Timeout on the PPS signal
	rjmp	CommonCWL200		;no
	std		TimeTicks,_0		;re-initialize 100Hz counter (INT0 is not executed)
	CALL	PPSMissingOn		;Turn on the PPS missing warning

#if FUNMODES
	ldd		r0,CurrPlayMode
	sbrc	r0,RUNCLOCKREVERSE
	rjmp	CommonCWL175
	rcall	UpdateSecs			;advance time. This will run 1/30 sec. slow
	rjmp	CommonCWL180
CommonCWL175:
	rcall	ReverseSecs
CommonCWL180:
#else
	rcall	UpdateSecs			;advance time. This will run 1/30 sec. slow
#endif
	cbr		G_FLAGS,(1<<fTimeOut)

CommonCWL200:
	sbrs	G_FLAGS,fFLASH		;Do we need to flash any fields
	rjmp	CommonCWL300
;
; Table Based Flashing
;
	CALL	ProcessFlashTbl
	cbr		G_FLAGS,(1<<fFLASH)	;Turn off Flash bit
;
CommonCWL300:
	sbrs	G_FLAGS,fMIN
	rjmp	CommonCWL500		;Skip tests for hours (fHR) boundary since it's only set
								;if fMIN is set
;
; Check rotate Grid Offset
;
	ldd		r16,RotationCountdown
	dec		r16
	brne	CommonCWL305
	CALL	RotateXYOffset
	ldi		r16,BURNINCOUNTDOWN	;initialize again
CommonCWL305:
	std		RotationCountdown,r16
;
#if SERIALIO
	lds		r0,Verbose
	tst		r0
	breq	CommonCWL310
	mov		r16,_1			;add CR/LF
	CALL	DisplayAVRDateTime
CommonCWL310:
#endif
	cbr		G_FLAGS,(1<<fMIN)	;turn off Minute Flag
CommonCWL330:
	ldd		r16,G2_FLAGS
	sbrs	r16,fHR		;Are we on an hour boundary
	rjmp	CommonCWL500
CommonCWL350:
	ldd		r16,G2_FLAGS
	cbr		r16,(1<<fHR)	;turn off Hour Flag
	std		G2_FLAGS,r16
CommonCWL500:
	ret
;
;
; ClockRefresh: draw one image on the CRT
;
; r17 used as the ScanTbl index, pointing to next entry to process
; r24, r22, r20, r18 are coordinates
; Z: current ScanTbl entry pointer
;
; No registers saved by this function. This is the
; responsibility of the caller.
;
; R24,R22 holds X0,Y0. R20,R18 holds X1,Y1
;
.def	X0				=	r24
.def	Y0				=	r22
.def	X1				=	r20
.def	Y1				=	r18
.def	XOffset			=	r11
.def	YOffset			=	r12
;
; include dac.asm here since it needs the definitions of XOffset and YOffset
;
.include ".\dac.asm"
;
ClockRefresh:
	ldd		r17,CurScanTblLen ;get current scan table length
	ldiw	X,ScanTbl		;compute pointer
ClockRefresh100:
	ld		ZL,X+			;get new coordinates pointer LO,HI
	ld		ZH,X+
	ld		XOffset,X+
	ld		YOffset,X+
	mov		r0,ZL			;NULL Ptr check
	or		r0,ZH
	breq	ClockRefresh2000
	rcall	DrawVectorTable	;preserve r17, X
ClockRefresh2000:
	dec		r17
	brne	ClockRefresh100
;
; now process any Strings, located in Flash
;
;	skip if CALIBRATION is active
;
RefreshStr:
	ldd		r0,ShowCalibration
	tst		r0
	brne	RefreshExit
	ldiw	X,ScanTbl_Str	;compute pointer
	ldi		r17,SCANSTRTABLEN	;count
	rcall	TextTableScan
;
; Process any Strings, located in SRam
;
	ldiw	X,ScanTbl_Str3	;compute pointer
	ldi		r17,SCANSSTRTABLEN	;count
	rcall	TextTableScanSRam
;
; Finished scanning the Display Table
;
RefreshExit:
#if CHRONOMETER
	ldsw	X,RefreshCnt
	adiw	X,1
	stsw	RefreshCnt,X
#endif
	BEAMOFF
	ret
;
; IN: r17, cnt   X, Table Ptr
;
TextTableScan:
	ld		ZL,X+			;get new coordinates pointer LO,HI
	ld		ZH,X+
	ld		XOffset,X+
	ld		YOffset,X+
	mov		r0,ZL			;skip NULL entry
	or		r0,ZH
	breq	TextTableScan200
	rcall	DrawTextStringfromFlash	;preserve r17,X
TextTableScan200:
	dec		r17
	brne	TextTableScan
	ret
;
; IN: r17, cnt   X, Table Ptr
;
TextTableScanSRam:
	ld		ZL,X+			;get new coordinates pointer LO,HI
	ld		ZH,X+
	ld		XOffset,X+
	ld		YOffset,X+
	mov		r0,ZL			;skip NULL entry
	or		r0,ZH
	breq	TextTableScanSRamL200
	rcall	DrawTextStringfromSRam	;preserver r17,X
TextTableScanSRamL200:
	dec		r17
	brne	TextTableScanSRam
	ret
;
; MenuRefresh: draw one image on the CRT
;
; r17 used as the ScanTbl index, pointing to next entry to process
;
; No registers saved by this function. This is the
; responsibility of the caller.
;
MenuRefresh:
	ldiw	X,MScanTbl			;table pointer
	ldi		r17,MENUSCANTABLEN	;count
	rcall	TextTableScan
;
; Finished scanning the Flash Text Pointers part of the Menu Table
; X points to next table entry: SRam Text Pointer
;
	mov		r17,_1
	rcall	TextTableScanSRam
;
#if CHRONOMETER
	lds		r17,RefreshCnt
	inc		r17
	sts		RefreshCnt,r17
#endif
	ret
;
; DrawVectorTable
;
; Z points to Flash based table of vector coordinates
;
; R24,R22 holds X0,Y0. R20,R18 holds X1,Y1
;
; start a new coordinates pair
;
; Need to preserve r17, X
;
DrawVectorTable:
	pushw	X
DrawVT100:
	lpm		X0,Z++			;get first byte from code space
DrawVTL150:
	tst		X0				;but skip a possible extra 0 byte here
	breq	DrawVT100
	cpi		X0,ESCAPE		;special
	breq	DrawVTChar		
	lpm		Y0,Z++			;get next byte from code space
;
; process coordinate pairs
;
DrawVTLoop:
	lpm		X1,Z++			;get first byte from code space
	tst		X1				;test the byte we got
	brne	DrawVTL200		;0 means "move to the next coordinate"
	rjmp	DrawVT100
DrawVTL200:
	cpi		X1,0xff			;0xff means "end of coordinates" for this table 
	breq	DrawVTExit
;
; Normal DrawVector using X,Y Data
;
	lpm		Y1,Z++			;get second byte from code space into r0
;
; Draw vector: r24:x0 r22:y0 r20:x1 r18:y1
;
	push	Y1				;save Y1,X1
	push	X1
	rcall	DrawVector		;draw a vector from X0,Y0 to X1,Y1
	pop		X0				;move X1,Y1 to X0,Y0
	pop		Y0
	rjmp	DrawVTLoop
;
; Support for characters in the vector tables.
; Used to display binary clock efficiently
;
; Format: ESCAPE VPOS CHAR
;
DrawVTChar:

	lpm		YOffset,Z++			;get VPOS from code space
	lpm		r16,Z++				;get char
	mov		r18,XOffset			;save
	rcall	DrawCharPreLoads	;preload for DrawChar
	rcall	DrawChar			;preserves r17,r18,X,Z
	mov		XOffset,r18			;restore
	lpm		X0,Z++				;get next byte from code space
	cpi		X0,0xff				;end of coordinates
	breq	DrawVTExit
	rjmp	DrawVTL150
;
; Finished scanning this table
;
DrawVTExit:
	popw	X
	ret
;
;	Vector Drawing Function: Bresenham's Algorithm
;
;	Arguments:
;	   r24 - X0-Start
;	   r22 - Y0-Start
;	   r20 - X1-End
;	   r18 - Y1-End
;
; When done, X0,Y0 and X1,Y1 are identical
;
; Don't use r17 (used in Refresh/MenuRefresh)
;
; X is used (to save Z) 
; Z holds the plot function to call
;
#if 0
void (*pfPlot)(x,y);
function line(x0, y0, x1, y1)
     boolean steep := abs(y1 - y0) > abs(x1 - x0)
     if steep then
	 	pfPlot = DACYXPOS;
		swap(x0, y0)
		swap(x1, y1)
     if x0 > x1 then
	 	pfPlot = DACXYPOS;
		swap(x0, x1)
		swap(y0, y1)
     int deltax := x1 - x0
     int deltay := abs(y1 - y0)
     int LineError := 0
     int YStep
     int y := y0
     if y0 < y1 then YStep := 1 else YStep := -1
     for x from x0 to x1
		pfPlot();
        LineError := LineError + deltay
        if 2LineError >= deltax then
             y := y + YStep
             LineError := LineError - deltax
#endif
.def	LineErrorL		=	r2
.def	LineErrorH		=	r3
.def	DeltaX			=	r4
.def	DeltaY			=	r5
.def	YStep			=	r6
DrawVector:
	
;	RCALL	SaveDACLODATA		;preset DACLODATA in r19
	movw	XL,ZL				;save Z
;     boolean steep := abs(y1 - y0) > abs(x1 - x0)
	ldiw	Z,DACXYPOS		;preset
	mov		r21,Y1
	sub		r21,Y0
	brcc	Line100
	neg		r21			;Take absolute value
Line100:
	mov		r16,X1
	sub		r16,X0
	brcc	Line200
	neg		r16			;Take absolute value
Line200:
	cp		r16,r21		;Inverted: if (Dx >= Dy) jmp
	brsh	Line300
	ldiw	Z,DACYXPOS
;     if steep then
;         swap(x0, y0)
;         swap(x1, y1)
;XOR swap algorithm
	eor		X0,Y0			;swap X0,Y0
	eor		Y0,X0
	eor		X0,Y0
	eor		X1,Y1			;swap X1, Y1
	eor		Y1,X1
	eor		X1,Y1
Line300:
;     if x0 > x1 then
;         swap(x0, x1)
;         swap(y0, y1)
	cp		X1,X0			;inverted: if (x1 >= x0) jmp
	brsh	Line400
	eor		X0,X1			;swap X0,X1
	eor		X1,X0
	eor		X0,X1
	eor		Y0,Y1			;swap Y0,Y1
	eor		Y1,Y0
	eor		Y0,Y1
Line400:
	mov		DeltaX,x1		;int deltax := x1 - x0
	sub		DeltaX,x0
	mov		DeltaY,Y1		;int deltay := abs(y1 - y0)
	sub		DeltaY,Y0
	brcc	Line500
	neg		DeltaY			;Take absolute value
Line500:
	clr		LineErrorL		;int LineError := 0
	clr		LineErrorH
	clr		YStep			;int YStep
	cp		Y0,Y1			;if y0 < y1 then ystep := 1 else ystep := -1
	brsh	Line600
	inc		YStep
	rjmp	Line750
Line600:
	dec		YStep
;     int y := y0
;     for x from x0 to x1
Line750:
	cp		X1,X0
	brlo	Line1100
	BEAMON
Line775:
#if DEBUG
	cpi		X1,0xff
	breq	PC
#endif
;		pfPlot();
	icall
;         LineError := LineError + deltay
	add		LineErrorL,DeltaY
	adc		LineErrorH,_0
Line997:
;         if 2LineError >= deltax then
	mov		r23,LineErrorL
	add		r23,LineErrorL
	cp		r23,DeltaX
	cpc		LineErrorH,_0
Line998:
	brlt	 Line1000
;             y := y + ystep
	add		Y0,YStep
;             LineError := LineError - deltax
	sub		LineErrorL,DeltaX
	sbc		LineErrorH,_0
Line1000:
	inc		X0
	cp		X1,X0			;inverted test
	brsh	Line775
;	brne	Line775
Line1100:
	BEAMOFF
	movw	ZL,XL
	ret
.undef	LineErrorL
.undef	LineErrorH
.undef	DeltaX
.undef	DeltaY
.undef	YStep
.undef	X0
.undef	Y0
.undef	X1
.undef	Y1
;
; Draw a String
;
; in Z: ptr to 0-terminated string
;
; Must preserve r17,X
;
DrawTextStringfromSRam:
	rcall	DrawCharPreLoads	;preload for DrawChar
DrawTS100:
	ld		r16,Z+				;get next Text Code from SRam
	tst		r16					;end of string
	breq	DrawTSL500			;done
	rcall	DrawChar
	rjmp	DrawTS100
DrawTSL500:
	ret

DrawTextStringfromFlash:
	rcall	DrawCharPreLoads	;preload for DrawChar
DrawTS600:
	lpm		r16,Z+				;get next Text Code from flash
	tst		r16					;end of string
	breq	DrawTSL700			;done
	rcall	DrawChar
	rjmp	DrawTS600
DrawTSL700:
	ret
;
; DrawChar Preloads
;
DrawCharPreLoads:
	ldi		r20,low(NewFonttbl*2)	;rebase was corrected for SPACE
	ldi		r21,high(NewFonttbl*2)
	ret
;
; DrawChar
;
; IN:	r16	-- char to draw. Valid range is 32..127 + HALFSPACE
;
; Preloads Expected: r20, r21
;
; USES: r0,r1,r6,r22,r24
; CharPixel uses: r3, r4 
;
; Free r5, r13,r23
;
; Preserves: r17, r18, X, Z
;
DrawChar:
	movw	r0,ZL			;save
	sbrc	r16,7			;bit 1 high is out of range
	rjmp	DrawCL900		;includes HALFSPACE
	subi	r16,FIRST_CHAR+1 ;rebase
	brcs	DrawCL850		;char was out of range, includes SPACE
#if !DRGcode
	BEAMON
#endif
	mov		ZL,r16			;compute 8 * char index
	clr		ZH
	lsl		ZL
	rol		ZH
	lsl		ZL
	rol		ZH
	lsl		ZL
	rol		ZH
	add		ZL,r20			;r20,r21 FontTable Ptr
	adc		ZH,r21
	ldi		r22,CHAR_HEIGHT	;First Y pos, Also Row Counter
DrawCL200:
	lpm		r6,Z+			;bits for current row
	clr		r24				;start x pos, 6 pixels per row
	sbrc	r6,7
	rcall	CharPixel
	inc		r24
	sbrc	r6,6
	rcall	CharPixel
	inc		r24
	sbrc	r6,5
	rcall	CharPixel
	inc		r24
	sbrc	r6,4
	rcall	CharPixel
	inc		r24
	sbrc	r6,3
	rcall	CharPixel
	inc		r24
	sbrc	r6,2
	rcall	CharPixel
	dec		r22				;next Y pos, row counter
	brne	DrawCL200		;finish all rows
DrawCL850:
	subi	r16,-(FIRST_CHAR+1)	;unbase
DrawCL900:
#if !DRGcode
	BEAMOFF
#endif


	movw	ZL,r0			;restore
	rcall	GetCharWidth	;r16 still char
	add		XOffset,r16
	ret
;
; GetCharWidth
;
; IN:	 r16 TextCode
; OUT:	 r16 width
;
GetCharWidth:
	cpi		r16,':'
	brne	GetCWL100
	ldi		r16,COLONCHARWIDTH
	rjmp	GetCWL800
GetCWL100:
	cpi		r16,HALFSPACE
	brne	GetCWL200
	ldi		r16,HALFSPACECHARWIDTH
	rjmp	GetCWL800
GetCWL200:
	cpi		r16,'-'
	brne	GetCWL300
	ldi		r16,DASHCHARWIDTH
	rjmp	GetCWL800
GetCWL300:
	ldi		r16,DEFCHARWIDTH
GetCWL800:
	ret

.undef	XOffset
.undef	YOffset
;
;
; a Second has passed. Make sure this function in ONLY called from ClockWorks
; Note:	this function may be called 5 t0 6 mSecs after the interrupt occurred
;		since we need to finish the Refresh() cycle. Not doing so results in
;		screen flicker.
;
; Register Use:
;
UpdateSecs:
	CALL	BlinkLed
;
;Update Time
;
	ldd		r17,AvrSecs		;get Seconds
	inc		r17				;update
	std		AvrSecs,r17
	cpi		r17,60			;check for overflow
	breq	UPDS_UpdMins	;TC1_DispAdrSec
;
; Update Display Pointers.
; Seconds Hands only unless DisplayRedrawFlg is set
;
	ldd		r16,DisplayRedrawFlg
	tst		r16
	breq	UPDS_DispAdrSec		;jump if fFORCE==FALSE
	std		DisplayRedrawFlg,_0	;Clear flag
	rjmp	UPDS_DispAdr	

UPDS_DispAdrSec:
	rcall	UpdateSecsHand
;
; Don't update numeric field if we're modifying the date
; This assumes that the numeric field is in DATE mode when modifying
;
	ldd		r16,CModState
	andi	r16,CMODDATE		;could use a skip here
	brne	UPDS_Out
	rcall	UpdClockNum
	rjmp	UPDS_Out
;
; a Minute has passed.
; Set Flag and keep time
;
UPDS_UpdMins:
	sbr		G_FLAGS,(1<<fMIN)	;set minutes flag
	std		AvrSecs,_0		;reset Seconds
	ldd		r17,AvrMins		;update minutes
	inc		r17
	std		AvrMins,r17
	cpi		r17,60			;check for overflow
	brlt	UPDS_UpdateRTC
;
; an Hour has passed.
;
	ldd		r17,G2_FLAGS
	sbr		r17,(1<<fHR)	;set Hours flag
	std		G2_FLAGS,r17
	std		AvrMins,_0		;reset minutes
	ldd		r17,AvrHrs		;update Hours
	inc		r17
	std		AvrHrs,r17
	cpi		r17,24			;check for overflow
	brlt	UPDS_UpdateRTC
;
; 24 hour overflow.
; Skip the RTC time update at the Hour transition so we don't miss a date transition
; (if 00 hr set before internal RTC transition from 24->00, then no date update)
; Manually inc date to get a proper display (don't know exactly when RTC date update happens)
;
	std		AvrHrs,_0
;
; if modifying the date, don't change
;
	ldd		r16,CModState
	andi	r16,CMODDATE
	brne	UPDS_L500
	CALL	AVRIncDate		;no need to update the RTC since it will follow shortly
UPDS_L500:
#if 1			;may remove later
;
; Finally, apply any DST adjustments (if enabled etc.)
; This seems unnecessary. Is there EVER a DST adjustment on a 24HR transition?
;
	ldd		r17,CurrPlayMode
	sbrc	r17,RUNCLOCKFAST		
	rcall	DSTRetroApply
#endif
	rjmp	UPDS_DispAdr
;
; Time has been updated. Now update the rtc which is the backup time,
;	unless we're in fast mode 
;
UPDS_UpdateRTC:
	ldd		r17,CurrPlayMode
	sbrc	r17,RUNCLOCKFAST		
	rjmp	UPDS_DispAdr
#if DEBUG
	sbrc	r17,RUNCLOCKREVERSE	;we should NOT get here in reverse mode
	rjmp	PC
#endif
	CALL	RTCSetTime
;
; if modifying the date, don't change the RTC, nor do DST correction.
; the menu code will do it explicitly.
;
	ldd		r16,CModState
	andi	r16,CMODDATE
	brne	UPDS_L700
	CALL	RTCReadDate		;sync date
;
; Finally, apply any DST adjustments (if enabled etc.)
;
UPDS_L600:
	rcall	DSTRetroApply
UPDS_L700:
;
; Update Display Pointers
;
UPDS_DispAdr:
	rcall	UpdateDisplay
UPDS_Out:
UPDSL097:
	ret

#if FUNMODES
;
; Set the clock back one second. Available for fun
; Mode only available in AVR Clock Master Mode
; RTC not updated.
;
ReverseSecs:
	RCALL	BlinkLed
	ldd		r16,ReloadPPSTimeOutCnt
	std		PPSTimeOutCnt,r16
;
;Reverse Time
;
	ldd		r17,AvrSecs		;get Seconds
	dec		r17				;update
	std		AvrSecs,r17
	cpi		r17,-1
	breq	RevSecsL100	;transition
;
; Update Display Pointers.
; Seconds Hands only unless DisplayRedrawFlg is set
;
	ldd		r0,DisplayRedrawFlg
	tst		r0
	breq	RevSecs_DispAdrSec	;jump if fFORCE==FALSE
	std		DisplayRedrawFlg,_0	;Clear flag
	rjmp	RevSecs_DispAdr	

RevSecs_DispAdrSec:
	rcall	UpdateSecsHand
	rcall	UpdClockNum
	rjmp	RevSecs_Out
;
; Minute boundary
;
RevSecsL100:
	ldi		r17,59
	std		AvrSecs,r17
	sbr		G_FLAGS,(1<<fMIN) ;set minutes flag
	ldd		r17,AvrMins	;update minutes
	dec		r17
	std		AvrMins,r17
	cpi		r17,-1
	brne	RevSecs_DispAdr	;check for transition
;
; an Hour has passed.
;
	ldi		r17,59
	std		AvrMins,r17		;reset minutes
	ldd		r17,G2_FLAGS
	sbr		r17,(1<<fHR)	;set Hours flag
	std		G2_FLAGS,r17
	ldd		r17,AvrHrs		;update Hours
	dec		r17
	std		AvrHrs,r17
	cpi		r17,-1
	brne	RevSecs_DispAdr	;check for transition
;
; a Day has passed.
;
	ldi		r17,23
	std		AvrHrs,r17
;
RevSecs_DispAdr:
	rcall	UpdateDisplay
RevSecs_Out:
	ret
#endif		;FUNMODES


;
; Reset the clock to the actual time after being in fun mode
; Only do this if both Fast and Reverse modes are off
;
SetActualTime:
	CALL	TestPlayModes
	tst		r16
	brne	SetATL100		;any FUN mode, exit	
	CALL	RTCReadTime
	CALL	RTCReadDate
	rcall	UpdateDisplay
SetATL100:
	ret
;
; Update internal Time Values from RTC and update display
;
UpdateTimeAndDisplay:
	CALL	RTCReadTime
UpdateDisplay:
	rcall	UpdateDialDigits
	rcall	UpdateClockHands
	rcall	UpdClockNum
	ret
;
; Update internal Date Values from RTC and update display
;
UpdateDateAndDisplay:
	CALL	RTCReadDate
	rjmp	UpdateDisplay
;
; Update Display Addresses for clock hands
; r16,r17,r18,r19 work registers
;
UpdateClockHands:
	rcall	UpdateSecsHand
	rcall	UpdateMinsHand
	rcall	UpdateHrsHand
	ret
;
; Update Display Addresses for Seconds hand
; r0,r1,r16,r17 work registers (hardware MUL)
;
; Each seconds hand position requires 5 bytes.
;
UpdateSecsHand:
UpdateSHL300:
	lds		r16,CurrDialVal	;Hands or Binary?
	cpi		r16,DIALBIN
	breq	UpdateSecsBinary
	ldiw	Z, SecPtrData*2	;Data for Seconds hand
	ldd		r16,AvrSecs		;current seconds, multiplicand
	ldi		r17,5			;multiply by 5
	mul		r16,r17			;result in r0:r1
	add		ZL,r0			;compute current seconds coordinates ptr
	adc		ZH,r1
	movw	r16,ZL
	ldiw	Z,ScanTbl_SecHand
	rjmp	UpdateDialCommon
UpdateSecsBinary:
	ldd		r16,AvrSecs		;Data for seconds
	CALL	binto2digits	;result in r16,r18
	ldiw	Z,Led3Tbl*2		;SecsHi column Vectors table
	rcall	GetPointer4Index
	ldiw	Z,ScanTbl_SecHand
	st		Z,r16
	std		Z+1,r17
	mov		r16,r18			;modulo 10 result
	ldiw	Z,Led4Tbl*2		;SecsLo column Vectors table
	rcall	GetPointer4Index
	ldiw	Z,ScanTbl_SecHand2
	rjmp	UpdateDialCommon
;
; Update Display Addresses for Minutes hand
; r0,r1,r16,r17 work registers (hardware MUL)
;
UpdateMinsHand:
	lds		r16,CurrDialVal	;Hands or Binary?
	cpi		r16,DIALBIN
	breq	UpdateMinsBinary
	ldiw	Z, MinPtrData*2	;Data for Minutes hand
	ldd		r16,AvrMins	;current minutes, multiplicand
	ldi		r17,9			;multiply by 9
	mul		r16,r17			;result in r0:r1
	add		ZL,r0			;compute current minutes coordinates ptr
	adc		ZH,r1
	movw	r16,ZL
	ldiw	Z,ScanTbl_MinHand
	rjmp	UpdateDialCommon
UpdateMinsBinary:
	ldd		r16,AvrMins		;Data for minutes
	CALL	binto2digits	;result in r16,r18
	ldiw	Z,Led3Tbl*2		;MinsHi column Vectors table
	rcall	GetPointer4Index
	ldiw	Z,ScanTbl_MinHand
	st		Z,r16
	std		Z+1,r17
	mov		r16,r18			;modulo 10 result
	ldiw	Z,Led4Tbl*2		;MinsLo column Vectors table
	rcall	GetPointer4Index
	ldiw	Z,ScanTbl_MinHand2
	rjmp	UpdateDialCommon
;
; Update Display Addresses for Hours hand
; r0,r1,r16,r17,r18 work registers (hardware MUL)
;
; Hours are counted on a 0..23 scale.
; compute Hours positions based on 12 minutes granularity:
; pos = (((hours % 12) * 60) + min) / 12
;
UpdateHrsHand:
	lds		r16,CurrDialVal	;Hands or Binary?
	cpi		r16,DIALBIN
	breq	UpdateHrsBinary
	ldiw	Z, HrPtrData*2	;Data for Hours hand
	ldd		r16,AvrMins	;minutes, dividend
	ldi		r17,12			;divisor, unchanged
	CALL	div08u			;div 12. result in r16
	push	r16				;save
	ldd		r16,AvrHrs		;hours, dividend
	CALL	div08u			;mod 12. result in r18
	mov		r16,r18			;multiplicand
	ldi		r18,5			;multiplier
	mul		r16,r18			;times 5. Result in r0:r1
	pop		r17				;minutes / 12
	add		r17,r0			;multiplicand
	ldi		r18,9
	mul		r17,r18			;times 9. Result in r0:r1
	add		Zl,r0			;compute current hours coordinates ptr
	adc		ZH,r1
	movw	r16,ZL
	ldiw	Z,ScanTbl_HrHand
	rjmp	UpdateDialCommon
UpdateHrsBinary:
	ldd		r16,AvrHrs		;Data for Hours Digits
	CALL	binto2digits	;result in r16,r18
	ldiw	Z,Led2Tbl*2		;HoursHi column Vectors table
	rcall	GetPointer4Index
	ldiw	Z,ScanTbl_HrHand
	st		Z,r16
	std		Z+1,r17
	mov		r16,r18			;modulo 10 result
	ldiw	Z,Led4Tbl*2		;HoursLo column Vectors table
	rcall	GetPointer4Index
	ldiw	Z,ScanTbl_HrHand2
;	rjmp	UpdateDialCommon	;FALL THROUGH

UpdateDialCommon:
	st		Z,r16
	std		Z+1,r17
	rcall	UpdateCModFlashEntry	;update StateFlashEntry
	ret
;
; Compute Numeric Display Buffer
;
UpdClockNum:
	ldiw	X,NumDispBuf
	lds		r19,CurrNumVal
;
; NULL string if no Numeric Display
;
	cpi		r19,NUMOFF
	breq	UCNL550
;
; date formatted string if DATE Display
;
	cpi		r19,NUMDATE
	brne	UCNL025

	CALL	FormatAVRDate
	rjmp	UCNL550
;
; Hexadecimal string if HEX Display
;
UCNL025:
	ldd		r16,AvrHrs		;Data for Hours Digits
	cpi		r19,NUMHEX
	breq	UCNL100			;join 24HR display
;
; Convert to AM/PM hours if active
;
UCNL050:
	cpi		r19,NUM12HR		;Are we displaying AM/PM
	brne	UCNL100
	movw	r0,ZL			;preserve Z
	ldiw	Z,AMPMMapping*2
	add		ZL,r16			;index
	adc		ZH,_0
	lpm		r16,Z			;get mapped value
	movw	ZL,r0
UCNL100:
	rcall	SetAsciiFromBin	;convert to (hexa)decimal
	ldi		r16,':'			;separator
	st		X+,r16
	ldd		r16,AvrMins		;Data for Minutes Digits
	rcall	SetAsciiFromBin	;convert to (hexa)decimal
	ldi		r16,':'			;separator
	st		X+,r16
	ldd		r16,AvrSecs		;Data for Seconds Digits
	rcall	SetAsciiFromBin	;convert to (hexa)decimal

	lds		r19,CurrNumVal
	cpi		r19,NUM12HR		;Are we displaying AM/PM
	brne	UCNL500			;No, clear AM/PM
	ldi		r16,HALFSPACE	;separator
	st		X+,r16
	ldi		r17,'A'			;Preset 'AM'
	ldd		r16,AvrHrs		;Data for Hours
	cpi		r16,12			;under 12, display A
	brlt	UCNL300
	ldi		r17,'P'			;Set 'PM'
UCNL300:
	st		X+,r17
	ldi		r17,'M'
	st		X+,r17
UCNL500:
#if CHRONOMETER
	lds		r0,ChronoEnabled
	tst		r0
	breq	UCNL550
	cpi		r19,NUM24HR		;r19 still CurrNumVal
	breq	UCNL525			;add digits
	cpi		r19,NUMHEX
	brne	UCNL550
;
; add space for the chronometer display
;
UCNL525:
	ldi		r17,':'
	st		X+,r17
	ldi		r17,SPACE
	st		X+,r17
	st		X+,r17
#endif
UCNL550:
	st		X+,_0

	rcall	SetNumDisplayBuf
;
; Now see if we need to build an Alternate Text Buffer for flashing
;
	ldd		r16,TextFlashField
	tst		r16
	breq	UCNL999			;no, done
;
	ldiw	Z,NumDispBuf
	ldiw	X,NumFlashDispBuf
	ldi		r17,DISPBUFLEN	;buffer length
UCNL600:
	ld		r16,Z+
	st		X+,r16
	dec		r17
	brne	UCNL600
	sbiw	X,DISPBUFLEN	;back to beginning (HRS/DAY)
	ldd		r16,TextFlashField ;1=HRS/DAYS, 2=MINS, 3=SECS, 4=MONTHS, 5=YEAR
	ldi		r17,SPACE
	cpi		r16,HourInString
	breq	UCNL800
	cpi		r16,MinInString
	breq	UCNL700
	cpi		r16,SecInString
	breq	UCNL650
	cpi		r16,YearInString
	breq	UCNL640
	cpi		r16,MonthInString
	breq	UCNL620
	rjmp	UCNL999
;
; flash Month field
;
UCNL620:
	adiw	X,3			;point to Month field
	st		X+,r17
	rjmp	UCNL800
UCNL640:
;
; flash Year field: 4 digits
;
	adiw	X,7			;Year field
	st		X+,r17
	st		X+,r17
	rjmp	UCNL800
UCNL650:				;Seconds field
	adiw	X,3
UCNL700:				;Minutes field
	adiw	X,3
UCNL800:
	st		X+,r17
	st		X+,r17
UCNL999:
	ret
;
; Set Flash Field and update Numeric Display Field
;
; IN: r16 - valid for Text Flash Field
;
SetNumFlashField:
	std		TextFlashField,r16
	rcall	UpdClockNum
	ret
;
; convert number to (hexa)decimal text
;
; IN: r16 number to convert
;	  X: buffer ptr to store text
;
SetAsciiFromBin:
	mov		r24,r16
	lds		r19,CurrNumVal
	cpi		r19,NUMHEX
	brne	SetAFBL100
;
; Hexadecimal Numeric Time Display
;
	RCALL	FormatHexadecimal
	rjmp	SetAFBL900
SetAFBL100:
;
; Decimal Numeric Time Display
;
	RCALL	FormatDecimal
SetAFBL900:
	ret

SetNumDisplayBuf:
	ldiw	X,ScanTbl_Str4	;Update Scan Table
	ldiw	Z,NumDispBuf	;needed twice
	st		X+,ZL
	st		X+,ZH
	rcall	ComputeSRamHPos	;center
	st		X+,r16			;Horizontal Position
	sts		CurrNumericHPos,r16	;save
	ldi		r16,NUMERICVPOS	;Vertical Position
	st		X+,r16
	ret

ClrNumDisplayBuf:
	ldiw	X,ScanTbl_Str4	;Update Scan Table
	st		X+,_0
	st		X+,_0
	st		X+,_0			;Horizontal Position
	st		X+,_0
	ret

#if CHRONOMETER
SetChronoDispBuf:
	lds		r0,ChronoEnabled
	tst		r0
	breq	ClrChronoDispBuf
	ldiw	X,ScanTbl_Str5	;Update Scan Table
	ldi		r17,NUMERICVPOS	;Vertical Position
#if MULTIAPP
	lds		r16,CurrAppMode
	cpi		r16,APPCLOCK
	breq	SetChronoDBL100
#if DEBUG
	cpi		r16,APPTERM
	breq	SetChronoDBL050
	rjmp	PC
SetChronoDBL050:
#endif
	ldiw	X,TermScanTbl_Str5	;Update Scan Table
	ldi		r17,TERMNUMERICVPOS	;Vertical Position
SetChronoDBL100:
#endif
	ldiw	Z,ChronoDispBuf
	st		X+,ZL
	st		X+,ZH
	lds		r16,CurrNumericHPos
	subi	r16,-NUMERICCHRONOPOS
	st		X+,r16			;Horizontal Position
	st		X+,r17
	ret

ClrChronoDispBuf:
	ldiw	X,ScanTbl_Str5	;Update Scan Table
#if MULTIAPP
	lds		r16,CurrAppMode
	cpi		r16,APPCLOCK
	breq	ClrChronoDBL100
#if DEBUG
	cpi		r16,APPTERM
	breq	ClrChronoDBL050
	rjmp	PC
ClrChronoDBL050:
#endif
	ldiw	X,TermScanTbl_Str5	;Update Scan Table
ClrChronoDBL100:
#endif
	st		X+,_0
	st		X+,_0
	st		X+,_0			;Horizontal Position
	st		X+,_0
	ret

UpdateChronoBuf:
	lds		r16,CurrNumVal
	cpi		r16,NUM24HR
	breq	UpdateCBL100
	cpi		r16,NUMHEX
	brne	UpdateCBL200
UpdateCBL100:
	ldiw	X,ChronoDispBuf
	ldd		r16,TimeTicks
	lsr		r16				;rescale to 0..100
;
; TimeTicks can overflow (get over 100) if the 1PPS signal is missing and
; we're faking the seconds transition after 1 1/30 a seconds
;
	cpi		r16,100
	brlt	UpdateCBL125
	subi	r16,100			;use value mod 100
	nop
UpdateCBL125:
	ldd		r0,CurrPlayMode
	sbrs	r0,RUNCLOCKREVERSE
	rjmp	UpdateCBL150
;
; clock is running reverse. display 100-x
;
	neg		r16
	subi	r16,-100
UpdateCBL150:
	rcall	SetAsciiFromBin
	st		X+,_0			;terminator
UpdateCBL200:
	ret
#endif
;
; Compute Vector for digit
;
; Arguments:
; In:
;	r16		digit
;	Z		ptr to code based ptr table
;
; Out:
;	r16,r17: vector ptr
;
GetPointer4Index:
	lsl		r16				;times 2
	add		ZL,r16
	adc		ZH,_0
	lpm		r16,Z+			;get lo(ptr)
	lpm		r17,Z			;get hi(ptr)
	lsl		r16				;multiply by 2 for AVR architecture
	adc		r17,r17
	ret
;
; Update the Dial Digits Ptr to use
;
UpdateDialDigits:
	lddw	Z,CurrDial12DigitsPtr	;preset to normal dial
	lds		r16,CurrNumVal
	cpi		r16,NUMOFF
	breq	UpdDialDgtsL50
	cpi		r16,NUMDATE
	brne	UpdDialDgtsL100	;show some form of Numeric Time -> use normal dial
;
; Numeric Display is either Off or Date. Use 24 hrs display
;
UpdDialDgtsL50:
	ldd		r16,AvrHrs		;current hours value
	cpi		r16,12
	brlo	UpdDialDgtsL100	;below 12
	lddw	Z,CurrDial24DigitsPtr
UpdDialDgtsL100:
	lds		r16,CurrDialVal
;
; check if a Binary Clock is being displayed
;
	cpi		r16,DIALBIN
	breq	UpdDialDgtsL200
	stsw	ScanTbl_Full,Z		;store dialdigits
UpdDialDgtsL200:
	ret
;
;	ValidateNumericDisplay
;
; OUT: r16 CurrNumVal
;
ValidateNumericDisplay:
	lds		r16,CurrNumVal
	cpi		r16,NUM12HR
	brlo	ValidateNDL100
	cpi		r16,NUMOFF+1
	brlo	ValidateNDL200
ValidateNDL100:
	ldi		r16,NUM12HR
	sts		CurrNumVal,r16
ValidateNDL200:
	ret
;
;	ValidateDialDisplay
;
; OUT: r16 CurrDialVal
;
ValidateDialDisplay:
	lds		r16,CurrDialVal
	cpi		r16,DIAL12HR
	brlo	ValidateDDL100
	cpi		r16,DIALMIN+1
	brlo	ValidateDDL200
ValidateDDL100:
	ldi		r16,DIAL12HR
	sts		CurrDialVal,r16
ValidateDDL200:
	ret
;
; SetNumericDisplay
;
; OUT:	r16 CurrNumVal
;
SetNumericDisplay:
	lds		r16,CurrNumVal
	push	r16
	cpi		r16,NUMOFF
	breq	SetNumL300
;
; default Display
;
	cpi		r16,NUM12HR
	breq	SetNumL100
	cpi		r16,NUM24HR
	breq	SetNumL200
	cpi		r16,NUMHEX
	breq	SetNumL200
	cpi		r16,NUMDATE
	breq	SetNumL100
#if DEBUG
	rjmp	PC					;invalid value
#endif
SetNumL100:
	rcall	ClrChronoDispBuf
	rcall	UpdClockNum			;will call SetNumDisplayBuf
	rjmp	SetNumL500
;
; Turn on 24 Hr or Hex display
;
SetNumL200:
	rcall	UpdClockNum			;will call SetNumDisplayBuf
	rcall	SetChronoDispBuf
	rjmp	SetNumL500
;
; Turn on Date display
;
SetNumL250:
	rcall	ClrChronoDispBuf
	rcall	UpdClockNum			;will call SetNumDisplayBuf
	rjmp	SetNumL500

;
; Turn off Numeric Display
;
SetNumL300:
	rcall	ClrNumDisplayBuf
	rcall	ClrChronoDispBuf
SetNumL500:
	pop		r16
	ret
;
; UpdateNumericDisplay. Called after a change
;
; in:	r16 New NumVal
;
UpdateNumericDisplay:
	sts		CurrNumVal,r16
#if DEBUG
	rcall	ValidateNumericDisplay
#endif
	rcall	SetNumericDisplay	;returns CurrNumVal in r16, validates
	rcall	UpdateNum_EEprom	;input is r16
	rcall	UpdateDialDigits
	rcall	UpdClockNum
	ret
;
; SetDialDisplay
;
; USES:	r16,r17,Z
;
SetDialDisplay:
	lds		r16,CurrDialVal
#if DEBUG
	rcall	ValidateDialDisplay
#endif
	cpi		r16,DIAL12HR
	breq	SetDialL100
#if DIAL24
	cpi		r16,DIAL24HR
	breq	SetDialL200
#endif
	cpi		r16,DIALROMAN
	breq	SetDialL300
	cpi		r16,DIALBIN
	breq	SetDialL400
;
; Must be DIALMIN
;
	ldiw	Z,DialDigitsMin*2		;Minimum Digits Display
	rjmp	SetDialL320				;join DIALROMAN
;
; Set DIAL12HR
;
SetDialL100:
	ldiw	Z,DialDigits*2			;Default  Digits Display
	rjmp	SetDialL320				;join DIALROMAN
#if DIAL24
;
; Set DIAL24HR
;
SetDialL200:
	ldiw	Z,DialDigits*2			;Default  Digits Display
	stdw	CurrDial12DigitsPtr,Z	;Current Dial Digits Display
	ldiw	Z,DialDigits24*2		;24 Hours Digits Display
	rjmp	SetDialL330				;join DIALROMAN
#endif
;
; set DIALROMAN
;
SetDialL300:
	ldiw	Z,DialDigitsRoman*2		;Roman Numerals Display
SetDialL320:
	stdw	CurrDial12DigitsPtr,Z	;Current Dial Digits Display
SetDialL330:
	stdw	CurrDial24DigitsPtr,Z	;Current 24 Hours Dial Digits Display
SetDialL350:
	ldiw	Z,ScanTblStatG+2		;Update status field locations
	sti		Z+,STATUSGHPOSANALOG
	sti		Z+,STATUSVPOSANALOG
	adiw	Z,2						;next field
	sti		Z+,STATUSBHPOSANALOG
	sti		Z+,STATUSVPOSANALOG
	adiw	Z,2						;next field
	sti		Z+,STATUSPHPOSANALOG
	sti		Z+,STATUSVPOSANALOG
	rcall	SetAnalogClockPtrs
	rjmp	SetDialL900				;done
;
; set DIALBIN
;
SetDialL400:
	ldiw	Z,ScanTblStatG+2		;Update status field location
	sti		Z+,STATUSGHPOSBIN
	sti		Z+,STATUSVPOSBIN
	adiw	Z,2						;next field
	sti		Z+,STATUSBHPOSBIN
	sti		Z+,STATUSVPOSBIN
	adiw	Z,2						;next field
	sti		Z+,STATUSPHPOSBIN
	sti		Z+,STATUSVPOSBIN

	rcall	SetBinaryClockPtrs
SetDialL900:
	rcall	UpdateDisplay
	ret
;
; UpdateDialDisplay. Called after a change
;
; in:	r16 New DialVal
;
UpdateDialDisplay:
	sts		CurrDialVal,r16
#if DEBUG
	rcall	ValidateDialDisplay
#endif
	rcall	SetDialDisplay
	lds		r16,CurrDialVal
	rcall	UpdateDial_EEprom	;input is r16
	rcall	UpdateDialDigits
	rcall	UpdateClockHands
	ret
;
; SetBinaryClockPtrs
;
; set Horizontal values for Binary Clock Display
; the Vertical values are included in the tables
;
SetBinaryClockPtrs:
	ldiw	Z,ScanTbl_SecHand+2	;X Offset
	ldi		r17,LedSHHPOS
	st		Z,r17
	adiw	Z,4					;next entry
	ldi		r17,LedSLHPOS
	st		Z,r17
	adiw	Z,4					;next entry
	ldi		r17,LedMHHPOS
	st		Z,r17
	adiw	Z,4					;next entry
	ldi		r17,LedMLHPOS
	st		Z,r17
	adiw	Z,4					;next entry
	ldi		r17,LedHHHPOS
	st		Z,r17
	adiw	Z,4					;next entry
	ldi		r17,LedHLHPOS
	st		Z,r17
;	adiw	Z,4					;next entry
;
; Disable DialDigits and Dial itself
;
	ldiw	Z,BinaryFrameData*2
	stsw	ScanTbl_Dial,Z
	clrsw	ScanTbl_Full
;
; Update Text Display Field, if enabled
;
	ldi		r17,TEXTBCLOCKVPOS
	rcall	SetUsrNameField
	ret
;
; Set Optional User Name Field
;
; IN: r17: vertical position (Y) to use
;
SetUsrNameField:
	ldiw	X,ScanTbl_Str3		;String Display Field
	lds		r0,UsrNameFlg		;0 means ON here
	tst		r0
	brne	SetUNFL100
	ldiw	Z,UsrNameBuf
	st		X+,ZL
	st		X+,ZH
	rcall	ComputeSRamHPos		;uses Z as input
	st		X+,r16				;result
	st		X+,r17				;Vertical Position
	rjmp	SetUNFL200
SetUNFL100:
	st		X+,_0
	st		X+,_0
SetUNFL200:
	ret
;
; ComputeSRamHPos
;	Compute Horizontal Pos necessary to center text
;
;	In:	Z ptr to SRam based text
;
;	Returns HPos in r16, 0 if overflow
;
;	Uses:	r18
;
ComputeSRamHPos:
	clr		r18					;width
CompSRamHPosL100:
	ld		r16,Z+
	tst		r16
	breq	CompSRamHPosL200
	rcall	GetCharWidth		;r16 is char, r16 return (0 if error)
	add		r18,r16
	brcc	CompSRamHPosL100	;no overflow
	clr		r16
	rjmp	CompSRamHposL300
CompSRamHPosL200:
	ser		r16					;(255-width)/2
	sub		r16,r18
	lsr		r16
CompSRamHposL300:
	ret

SetAnalogClockPtrs:
;
; Clear Scan Table Ptrs and Horizontal values for Analog Clock Display
;
	ldi		r16,6
	ldiw	Z,ScanTbl_SecHand
SetAnalogL100:
	st		Z+,_0				;Scan Tbl Ptr Lo
	st		Z+,_0				;Scan Tbl Ptr Hi
	st		Z+,_0				;Scan Tbl Ptr X Offset
	adiw	Z,1					;next entry
	dec		r16
	brne	SetAnalogL100
;
; Update Text Display Field
;
	clrsw	ScanTbl_Str3		;String Display Field
;
; Update Text Display Field, if enabled
;
	ldi		r17,TEXTACLOCKVPOS
	rcall	SetUsrNameField
;
; Restore DialDigits and Dial itself
;
	ldiw	Z,DialData*2		;Dial Data
	stsw	ScanTbl_Dial,Z
	rcall	UpdateDialDigits
	ret


.include "./DSTime.asm"
.include "./gps.asm"
.include "./ChangeMenu.asm"
;
; End of Menu Action Functions
;
#if MULTIBAUD
;
; Set Baud Rate: uart and indicator
; IN:	r16		0 based baudrate code
;
; USES: r16,r17,r18,r19,Z
;
SetBaudVal:
#if DEBUG
	cpi		r16,(BAUDVAL14400 - BAUDVALBASE + 1) 
	brcs	SetBVL100				;in range
	rjmp	PC						;error
SetBVL100:
#endif
	ldiw	Z,MapBaudVal*2			;4 bytes each
	lsl		r16						;times 2
	add		ZL,r16					;compute ptr
	adc		ZH,_0
	lpm		r18,Z+					;get AVR value
	lpm		r19,Z+
;
;	Change baud rate
;
#if SERIALIO
	CALL	UARTFlushPendingOutput	;uses r16
#endif
	CALL	UART_SetBaudRate		;r18/r19 input
	ret
#endif

;
; InitLed
;
InitLed:
	sbi		LED_DDR,LED_BIT
	cbi		LED_PORT,LED_BIT	;turn off
#if Mega32
	sbi		LED_DDR,LED2_BIT
	cbi		LED_PORT,LED2_BIT	;turn off
	sts		LedStatus,_0
#endif
	ret
;
; BlinkLed. Try to save space here.
;
BlinkLed:
	lds		r16,LedDisabled
#if (Mega168 | Mega328p)
	sbrs	r16,0				;no blinking if bit 0 is TRUE
	sbi		LED_PIN,LED_BIT		;toggle the LED
#elif Mega32
	tst		r16
	brne	BlinkLedOut
	lds		r16,LedStatus
	tst		r16
	breq	BlinkLedOn
	cbi		LED_PORT,LED_BIT	;turn off
	sts		LedStatus,_0
	rjmp	BlinkLedOut
BlinkLedOn:
	sbi		LED_PORT,LED_BIT	;turn on
	sts		LedStatus,_1
BlinkLedOut:
#endif
	ret
;
; TurnOffLed
;
TurnOffLed:
	cbi		LED_PORT,LED_BIT
	ret

#if Mega32
;
; TurnOnLed2
;
TurnOnLed2:
	sbi		LED_PORT,LED2_BIT
	ret
;
; TurnOff Led2
;
TurnOffLed2:
	cbi		LED_PORT,LED2_BIT
	ret
#endif

#if DEBUG
;
; DBGTurnLedOn
;
DBGTurnLedOn:
	sbi		LED_PORT,LED_BIT
	ret
;
; DBGTurnLedOff
;
DBGTurnLedOff:
	cbi		LED_PORT,LED_BIT
	ret
#endif

;
; InitSwitch
;
InitSwitch:
	cbi		SW_DDR,SW_BIT		;set as input
	sbi		SW_PORT,SW_BIT		;enable pull-up resistor
	cbi		SW_DDR,SW2_BIT		;set as input
	sbi		SW_PORT,SW2_BIT		;enable pull-up resistor
	ret
;
; The RTC reports a Low Battery (VL flag). This only has meaning when 
; the AVR starts up (battery should not go low while running from 5V)
;
BatteryWarning:
	ldd		r16,BatteryLow
	cpi		r16,0xff				;was this reported earlier
	brne	BatWarningL100			;yes, ignore
	std		BatteryLow,_1			;set low battery flag
	ldsw	X,BatFlashPtr			;FlashTbl entry
	mov		r16,XL					;skip if already active
	or		r16,XH					;this works since we only set this once
	brne	BatWarningL100
	ldi		r16,FLASHPERSEC*60		;60 seconds max 'B' warning
	rcall	BatteryStartFlashing
	stsw	BatFlashPtr,X			;needed to turn Battery warning signal off
BatWarningL100:
	ret
;
; Display a flashing 'B' for n seconds
;
; IN r16 -- number of seconds to flash
;
; OUT X -- FlashPtr
;
BatteryStartFlashing:
	ldiw	X,ScanTblStatB
	ldiw	Z,CharB*2				;'B'
	rcall	AddFlashItemNoAlt
	ret
;
; Disable the PPS timeout
;
DisablePPSTimeOut:
	std		PPSTimeOutCnt,_0
	ret
;
; Enable the PPS timeout
;
; OUT: r16 new PPSTimeOutCnt
;
EnablePPSTimeOut:
	ldd		r16,ReloadPPSTimeOutCnt ;reinitialize tick counter for timeout
	std		PPSTimeOutCnt,r16
	ret
;
; The PPS signal is missing. Turn on the warning field.
; This warning overrules the 'B' warning
;
; No warning in Fast Mode since we use the timeout to run fast!
;
PPSMissingOn:
	ldd		r16,CurrPlayMode			;are we in Fast Mode
	sbrc	r16,RUNCLOCKFAST
	ret								;yes, ignore
	lddw	X,PPSFlashPtr			;FlashTbl entry
	mov		r16,XL					;skip if already active
	or		r16,XH
	brne	PPSMissingL100
	ldi		r16,0xff				;forever
	rcall	PPSStartFlashing
	stdw	PPSFlashPtr,X			;needed to turn PPS missing signal off
PPSMissingL100:
	ret
;
; Display a flashing 'P' for n seconds
;
; IN r16 -- number of seconds to flash
;
; OUT X -- FlashPtr
;
PPSStartFlashing:
	ldiw	X,ScanTblStatP
	ldiw	Z,CharP*2				;'P'
	rcall	AddFlashItemNoAlt
	ret
;
; The PPS signal is back. Turn off the warning field
;
PPSMissingOff:
	lddw	X,PPSFlashPtr			;FlashTbl entry
	rcall	RemoveFlashItem
	std		PPSFlashPtr,_0			;clear ptr
	std		PPSFlashPtr+1,_0
	ret
;
; Rotate the X/Y offset values
;
; Uses: r16
;
RotateXYOffset:
	ldd		r16,RotationIndex
	cpi		r16,0xff			;disabled flag
	breq	RotL100
	movw	r0,ZL				;save
	ldiw	Z,RotationTable*2
	inc		r16
	andi	r16,0b00000111		;index % 8
	std		RotationIndex,r16
	lsl		r16					;Compute table ptr
	add		ZL,r16
	adc		ZH,_0
	lpm		GXOffset,Z+			;get new values
	lpm		GYOffset,Z
	movw	ZL,r0				;restore
RotL100:
	ret
;
; Enable Grid Rotation
; Don't enable RotationIndex if Burn-in Protection is disabled
;
; Uses: r0
;
EnableRotation:
	lds		r0,BurnInDisabled
	tst		r0
	brne	EnableRotationL100
	lds		r0,SavedRotationIndex
	std		RotationIndex,r0
	rcall	RotateXYOffset
EnableRotationL100:
	ret
;
; Disable Grid Rotation
;
; Uses: r16
;
DisableRotation:
	ldd		r16,RotationIndex
	sts		SavedRotationIndex,r16
	ser		r16
	std		RotationIndex,r16
	clr		GXOffset
	clr		GYOffset
	ret
;
; Set RotationIndex
;
; argument: r16 (0: ON or 1: OFF).
;
; Bug fix.	Argument removed as of V3.5a
;
; uses: r0
;
SetRotation:
;
; do not update RotationIndex if Calibration is on
;
	ldd		r0,ShowCalibration
	tst		r0
	brne	SetRotationL200		;skip
;
; Bug fixed in V3.5a. R16 argument was not always loaded
;
	lds		r16,BurnInDisabled
	tst		r16					;flag value
	breq	SetRotationL100		;now not disabled
	rcall	DisableRotation
	rjmp	SetRotationL200
SetRotationL100:
	rcall	EnableRotation		;Turn On Rotation
SetRotationL200:
	ret
;
; Various EEprom Update functions.
;
; IN: r16 -- new value
;
UpdateDial_EEprom:				;r16 is parameter
	ldi		XL,EEPROM_DIAL
	rjmp	Update_EEprom

UpdateLED_EEprom:				;r16 is parameter
	ldi		XL,EEPROM_LED
	rjmp	Update_EEprom

UpdateBurn_EEprom:				;r16 is parameter
	ldi		XL,EEPROM_BURN
	rjmp	Update_EEprom

UpdateNum_EEprom:				;r16 is parameter
	ldi		XL,EEPROM_NUMERIC
	rjmp	Update_EEprom

UpdateDSTMode_EEprom:			;r16 is parameter
	ldi		XL,EEPROM_DSTMODE
	rjmp	Update_EEprom

UpdateDSTFVec_EEprom:			;r16 is parameter
	ldi		XL,EEPROM_DSTFVEC
	rjmp	Update_EEprom

UpdateDSTBVec_EEprom:			;r16 is parameter
	ldi		XL,EEPROM_DSTBVEC
	rjmp	Update_EEprom

UpdateVerb_EEprom:				;r16 is parameter
	ldi		XL,EEPROM_VERBOSE
	rjmp	Update_EEprom

UpdateGPS_EEprom:
	ldi		XL,EEPROM_GPSMODE
	rjmp	Update_EEprom

UpdateGPSOffset_EEprom:
	ldi		XL,EEPROM_GPSOFFSET
	rjmp	Update_EEprom
;
; Mark RTC as initialized
;
UpdateRTCINIT_EEprom:
	ldi		XL,EEPROM_RTCINIT
	ldi		r16,1
	rjmp	Update_EEprom
;
UpdateUsrNameFlg:
	ldi		XL,EEPROM_USRNAMEFLG
	rjmp	Update_EEprom
;
UpdateChrono_EEprom:
	ldi		XL,EEPROM_CHRONO
	rjmp	Update_EEprom
;
UpdatePPSTrigger_EEprom:
	ldi		XL,EEPROM_PPSTRIG
	rjmp	Update_EEprom
;
UpdateBaudrate_EEPprom:
	ldi		XL,EEPROM_BAUDRATE
	rjmp	Update_EEprom
;
UpdateStartApp_EEPprom:
	ldi		XL,EEPROM_STARTAPP
	rjmp	Update_EEprom
;
; Common code between most EEprom Update functions
'
Update_EEprom:
#if DEMOMODE
	lds		r0,DemoEnabled		;No Update in Demo Mode
	tst		r0
	brne	UpdateEEL100
#endif
	clr		XH
	out		EEDR,r16			; EEDR = r16
	ldi		R17,6 				; write EEPROM
	rcall	EepromTalk
UpdateEEL100:
	ret
;
; Read Configuration Data as a block on startup
;
ReadEEConfiguration:
	ldiw	X,EEConfigDataEnd
	ldiw	Z,EEConfigData
	sub		XL,ZL
	sbc		XH,ZH
#if DEBUG
	tst		XH					;better be 0
	breq	ReadEEL100
	rjmp	PC
ReadEEL100:
#endif
	mov		r18,XL				;count
	ldiw	X,EEPROM_LED		;Configuration Start Area in EEPROM
	ldi		r17,1 				;read EEPROM
ReadEEL200:
	rcall	EepromTalk			;result in r16
	cpi		r16,0xff			;0xFF means not-initialized
	brne	ReadEEL300
	clr		r16
ReadEEL300:
	st		Z+,r16				;store
	dec		r18
	brne	ReadEEL200
	ret
;
; Read UsrName from EEprom and store in UsrNameBuf
;
ReadEEUsrName:
	ldiw	Z,UsrNameBuf
	lds		r16,UsrNameCnt		;get count
	tst		r16					;if 0, use default name
	breq	ReadEEUNL300
#if DEBUG
	cpi		r16,MAXUSRNAMELEN+1	;sanity check
	brsh	ReadEEUNL300		;invalid, use default
#endif
	mov		r18,r16
	ldiw	X,EEPROM_USRNAME	;Load UsrName area from EEPROM
	ldi		r17,1				;read EEPROM
ReadEEUNL100:
	rcall	EepromTalk			;result in r16
#if DEBUG
	cpi		r16,0xff			;should not happen
	brne	ReadEEUNL200
	ldi		r16,' '
ReadEEUNL200:
#endif
	st		Z+,r16
	dec		r18
	brne	ReadEEUNL100
	rjmp	ReadEEUNL400
ReadEEUNL300:
	movw	XL,ZL
	ldiw	Z,MsgPszLogo*2		;Dutchtronix string
	ldi		r18,11				;length
	sts		UsrNameCnt,r18
ReadEEUNL350:
	lpm		r16,Z+				;copy
	st		X+,r16
	dec		r18
	brne	ReadEEUNL350
	sts		UsrNameDflt,_1		;default was used
ReadEEUNL400:
	ret
;
; Store UsrNameBuf in EEprom
;
WriteEEUsrName:
#if DEMOMODE
	lds		r0,DemoEnabled		;No Update in Demo Mode
	tst		r0
	brne	WriteEEUNL200
#endif
	lds		r0,UsrNameDflt		;Using the default name?
	tst		r0
	brne	WriteEEUNL200		;yes, skip EEProm update
	lds		r16,UsrNameCnt		;get count
	tst		r16					;if 0, means ReadEEUsrName was never called
	brne	WriteEEUNL10		; or user selected an empty name. Save a single space
	ldi		r16,SPACE
	sts		UsrNameBuf,r16
;
; bug fix in V3.5a. Empty user name was unterminated
;
	sts		UsrNameBuf+1,_0		;terminate
	mov		r16,_1
	sts		UsrNameCnt,r16
WriteEEUNL10:
	mov		r18,r16
	ldiw	X,EEPROM_USRNAMECNT	;Store UsrNameCNT area in EEPROM
	out		EEDR,r16			; EEDR = r16
	ldi		r17,6				;update EEPROM
	rcall	EepromTalk			;write Cnt (r16)
	ldiw	X,EEPROM_USRNAME	;Store UsrName area in EEPROM
	ldiw	Z,UsrNameBuf
WriteEEUNL100:
	ld		r16,Z+				;get char
	out		EEDR,r16			; EEDR = r16
	rcall	EepromTalk			;result in r16
	dec		r18
	brne	WriteEEUNL100

WriteEEUNL200:
	ret
;
; Clear all Configuration Data
;
ClearEEConfiguration:
	ldiw	X,EEConfigDataEnd
	ldiw	Z,EEConfigData
	sub		XL,ZL
	sbc		XH,ZH
#if DEBUG
	tst		XH				;better be 0
	breq	ClrEEL100
	rjmp	PC
ClrEEL100:
#endif
	mov		r18,XL			;count
	ldiw	X,EEPROM_LED	;Configuration Start Area in EEPROM
	ldi		r17,6 			;write EEPROM
ClrEEL200:
	ser		r16				;0xff init value
	out		EEDR,r16		;write new value to EEDR
	rcall	EepromTalk
	st		Z+,_0			;Reset memory too
	dec		r18
	brne	ClrEEL200
	ret
;
; (Hexa)Decimal Formatting of binary value in r24
; Store in X pointed buffer, increment X
; Only values in r24 < 100 supported (if Decimal)
;
FormatDecimal:
#if DEBUG
	cpi		r24,100
	brsh	DDL100
#endif
	rcall	bintobcd
FormatHexadecimal:
	push	r24
	swap    r24
	andi    r24,0X0F
	rcall   Hex2Asc
	st		X+,r24
	pop     r24
	andi    r24,0X0F
	rcall   Hex2Asc
	st		X+,r24
DDL100:
	ret
;
; Format Current Date in Buffer (X)
;
; Buffer overflow checked at code time (fixed format)
;
FormatAVRDate:
	ldiw	Z,AVRTimeDateBlk
;	rcall	FormatDatefromZ		;FALL THROUGH
;	ret
;
; This function assume that the Z pointer has been loaded with a
; pointer to the desired time block (Avr or UTC)
;
FormatDatefromZ:
	ldd		r24,pDay			;Display Day
	RCALL	FormatDecimal
	ldi     r24,'-'
	st		X+,r24
	ldd		r24,pMonth			;Display Month
	movw	r0,ZL				;save
	ldiw	Z,MonthsText*2
	dec		r24					;rebase 0..11
	lsl		r24					;times 2
	lsl		r24					;times 4
	add		ZL,r24				;compute table ptr
	adc		ZH,_0
	lpm		r24,Z+				;three chars
	st		X+,r24
	lpm		r24,Z+
	st		X+,r24
	lpm		r24,Z+
	st		X+,r24
	ldi     r24,'-'
	st		X+,r24
	movw	ZL,r0				;restore
	ldi		r24,'2'				;Display Year
	st		X+,r24
	ldi		r24,'0'
	st		X+,r24
	ldd		r24,pYear
	RCALL	FormatDecimal
	ret

;
; Format Current Time in Transmit Buffer (X)
;
; Buffer overflow checked at code time (fixed format)
;
; Duplicate code. See UpdClockNum
;
FormatAVRTime:
	ldiw	Z,AVRTimeDateBlk
;	rjmp	FormatTimefromZ		;FALL THROUGH

FormatTimefromZ:
	ldd		r24,pHrs			;Display Hours
	RCALL	FormatDecimal
	ldi     r24,':'
	st		X+,r24
	ldd		r24,pMins			;Display Minutes
	RCALL	FormatDecimal
	ldi     r24,':'
	st		X+,r24
	ldd		r24,pSecs			;Display Seconds
	RCALL	FormatDecimal
	ret


;
; AdvanceTimeOneHr --
;	Increment the current AvrHr as part of Daylight Saving Time
;
; Z - ptr to AVRTimeDateBlk
;
AdvanceAVRTimeOneHr:
#if DEBUG
	cpi		ZL,low(AVRTimeDateBlk)
	brne	AdvanceAVRTL100
	cpi		ZH,high(AVRTimeDateBlk)
	breq	AdvanceAVRTL200
AdvanceAVRTL100:
	rjmp	PC
AdvanceAVRTL200:
#endif
;	ldiw	Z,AVRTimeDateBlk
	ldd		r17,AvrHrs		;update Hours
	inc		r17
	std		AvrHrs,r17
	cpi		r17,24			;check for overflow
	brlt	AdvanceTOH900
	std		AvrHrs,_0
	rcall	IncDatefromZ
	RCALL	RTCSetDate		;update RTC
AdvanceTOH900:
	RCALL	RTCSetTime		;update RTC
	ret
;
; RetreatTimeOneHr --
;	Decrement the current AvrHr as part of Daylight Saving Time
;
RetreatAVRTimeOneHr:
#if DEBUG
	cpi		ZL,low(AVRTimeDateBlk)
	brne	RetreatAVRTL100
	cpi		ZH,high(AVRTimeDateBlk)
	breq	RetreatAVRTL200
RetreatAVRTL100:
	rjmp	PC
RetreatAVRTL200:
#endif
;	ldiw	Z,AVRTimeDateBlk
	ldd		r17,AvrHrs		;update Hours
	dec		r17
	std		AvrHrs,r17
	cpi		r17,-1			;check for overflow
	brne	RetreatTOH900
	ldi		r17,23
	std		AvrHrs,r17
	rcall	DecDatefromZ
	RCALL	RTCSetDate		;update RTC
RetreatTOH900:
	RCALL	RTCSetTime		;update RTC
	ret
;
; Increment Date
;
AVRIncDate:
	ldiw	Z,AVRTimeDateBlk
	rcall	IncDatefromZ
	ret
;
; Decrement Date
;
AVRDecDate:
	ldiw	Z,AVRTimeDateBlk
	rcall	DecDatefromZ
	ret
;
; Increment Date
;
IncDatefromZ:
	rcall	IncDayfromZ
	cp		r16,_1				;wrapped?
	brne	IncDateL900			;no
	rcall	IncMonthfromZ
	cp		r16,_1				;wrapped?
	brne	IncDateL900
	rcall	IncYearfromZ
IncDateL900:
	ret
;
; Decrement Date
;
DecDatefromZ:
	ldd		r16,pDay			;dec current Day
	dec		r16
	std		pDay,r16
	brne	DecDateL800			;no wrap
;
; wrap day: compute new day later
;
	ldd		r16,pMonth			;dec current Month
	dec		r16
	std		pMonth,r16
	brne	DecDateL100			;no month wrap
	ldi		r16,12				;set to December
	std		pMonth,r16
	ldd		r16,pYear			;dec current Year
	dec		r16
	std		pYear,r16			;ignore wrap
;
; now get the last day of the new month in new year
;
DecDateL100:
	rcall	GetDaysInMonthfromZ
	std		pDay,r17
DecDateL800:
	ret
;
; compute number of days present in Avrmonth.
;
; OUT: r17 number of days in AvrMonth
;
GetAVRDaysInMonth:
	pushw	Z	
	ldiw	Z,AVRTimeDateBlk
	rcall	GetDaysInMonthfromZ
	popw	Z	
	ret
;
; compute number of days present in current month.
; Use the "MonthsText" table.
; watch for leap years
;
; OUT: r17 number of days in current (AvrMonth) month
;
GetDaysInMonthfromZ:
	pushw	X
	push	r16
	ldd		r16,pMonth			;Get Month
	ldiw	X,MonthsText*2
	dec		r16					;rebase 0..11
	lsl		r16					;times 2
	lsl		r16					;times 4
	add		XL,r16				;compute table ptr
	adc		XH,_0
	adiw	X,3					;point to length
	pushw	Z
	movw	ZL,XL
	lpm		r17,Z				;get #of days in month
	popw	Z
	cpi		r17,28				;February?
	brne	GetDaysIML100		;no
	ldd		r16,pYear			;get current Year
	andi	r16,0x03			;mod 4. If != 0, no leap year
	brne	GetDaysIML100
;
; Possible leap year.
;
; Gregorian Calendar:
; A year will be a leap year if it is divisible by 4 but not by 100.
; If a year is divisible by 4 and by 100, it is not a leap year
; unless it is also divisible by 400.
; Thus years such as 1996, 1992, 1988 and so on are leap years because
; they are divisible by 4 but not by 100. For century years, the 400
; rule is important. Thus, century years 1900, 1800 and 1700 while all
; still divisible by 4 are also exactly divisible by 100. As they are
; not further divisible by 400, they are not leap years.
; The year 2000 is the only year of interest for us (we support 2000..2099)
; and it IS a leap year. Therefore no special test needed.
;
	ldi		r17,29				;leap year, return 29
GetDaysIML100:
	pop		r16
	popw	X
	ret
;
; call this function before setting the Date in the RTC
;
; IN: r16 day to check
;
; OUT: r16 == 0 error, else input value
;
CheckAVRDaysInMonth:
	ldiw	Z,AVRTimeDateBlk
	rcall	CheckDaysInMonthfromZ
	ret
;
; IN: r16 day to check
;
; OUT:	r16 == 0 error, else input value
;		r17 Days in Month + 1
;
CheckDaysInMonthfromZ:
	rcall	GetDaysInMonthfromZ		;in r17
	inc		r17
	cp		r16,r17
	brlo	CheckDIML100
	clr		r16
CheckDIML100:
	ret
;
; Increment Day
; Use the current month for limit check.
;
AVRIncDay:
	ldiw	Z,AVRTimeDateBlk
	rcall	IncDayfromZ
	ret
;
; IN: r16 (when setting)
;
AVRIncMonth:
	ldiw	Z,AVRTimeDateBlk
	rcall	IncMonthfromZ
	ret
;
; 00-99 are valid years
;
; IN: r16 (when setting)
;
AVRIncYear:
	ldiw	Z,AVRTimeDateBlk
	rcall	IncYearfromZ
	ret

AVRSetDay:
	ldiw	Z,AVRTimeDateBlk
	rjmp	AVRSetDayfromZ

AVRSetMonth:
	ldiw	Z,AVRTimeDateBlk
	rjmp	AVRSetMonthfromZ

AVRSetYear:
	ldiw	Z,AVRTimeDateBlk
	rjmp	AVRSetYearfromZ
;
; Increment Day
; Use the month in current TimeDate block for limit check.
;
IncDayfromZ:
	ldd		r16,pDay
	inc		r16
AVRSetDayfromZ:
	tst		r16					;user input can be anything
	breq	AVRIncDayL50
	rcall	CheckDaysInMonthfromZ ;r16 input
	tst		r16
	brne	AVRIncDayL100		;ok
AVRIncDayL50:
	mov		r16,_1				;wrap
AVRIncDayL100:
	std		pDay,r16
	ret
;
; IN: r16 (when setting)
;
IncMonthfromZ:
	ldd		r16,pMonth
	inc		r16
AVRSetMonthfromZ:
	tst		r16					;user input can be anything
	breq	AVRIncMoL50
	cpi		r16,13
	brlo	AVRIncMoL100
AVRIncMoL50:
	mov		r16,_1					;wrap
AVRIncMoL100:
	std		pMonth,r16
	ret
;
; 00-99 are valid years
;
; IN: r16 (when setting)
;
IncYearfromZ:
	ldd		r16,pYear
	inc		r16
AVRSetYearfromZ:
	cpi		r16,100
	brlo	AVRIncYRL200
	clr		r16
AVRIncYRL200:
	std		pYear,r16
	ret
;
; Set AVR Time. Binary input in r24
; Uses r16,r17
;
AVRSetSecs:
	mov		r16,r24
	cpi		r16,60				;sanity check
	brsh	AVRSSL100
	std		AvrSecs,r24
	ret
AVRSSL100:
	clr		r16
	ret

AVRSetMins:
	mov		r16,r24
	cpi		r16,60				;sanity check
	brsh	AVRSML100
	std		AvrMins,r24
	ret
AVRSML100:
	clr		r16
	ret

AVRSetHours:
	mov		r16,r24
	cpi		r16,24				;sanity check
	brsh	AVRSHL100
	std		AvrHrs,r24
	ret
AVRSHL100:
	clr		r16
	ret
;
; Turn Calibration Display Off
;
CalibrationOff:
#if DEBUG		;consistency check
	ldd		r16,ShowCalibration
	tst		r16
	breq	CalOffL100
	rjmp	PC
CalOffL100:
#endif
	rcall	StopRTCClockOut		;stop RTC Clockout Signal
	rcall	EnableRotation
;
; Restore the Display Scan Table.
;
	ldi		r17,SCANTABLEN			;restore Value
	std		CurScanTblLen,r17
	CALL	SetDialDisplay
	ret
;
; Turn Calibration Display On
;
CalibrationOn:
#if DEBUG		;consistency check
	ldd		r16,ShowCalibration
	tst		r16
	brne	CalOnL100
	rjmp	PC
CalOnL100:
#endif
#if FUNMODES
	rcall	DisablePlayModes
#endif
	rcall	StartRTCClockOut		;start RTC Clockout Signal
	rcall	DisableRotation
;
; Modify the Display Scan Table.
;
	std		CurScanTblLen,_1		;just one Entry if Calibration Pattern
	ldiw	Z,CalibrationData*2		;Calibration Data
	stsw	ScanTbl_Dial,Z
	ret
;
; Toggle Calibration Pattern Display
;
ToggleCalibration:
	ldd		r16,ShowCalibration		;Current Mode
	eor		r16,_1					;toggle
;
; FALL THROUGH
;
; UpdateCalibration
;
; IN r16 -- new calibration mode
;
UpdateCalibration:
	std		ShowCalibration,r16		;Store new Mode
	tst		r16						;on or off?
	breq	UpdateCL100
	rcall	CalibrationOn
	rjmp	UpdateCL200
UpdateCL100:
	rcall	CalibrationOff			;just one of these call will do anything
UpdateCL200:
	rjmp	clctail2


;
; Toggle PPSTrigger
;
TogglePPSTrigger:
	lds		r16,PPSTrigger			;Current Mode
	eor		r16,_1					;toggle
;
; FALL THROUGH
;
; UpdatePPSTrigger
;
; IN r16 -- new PPSTrigger mode
;
UpdatePPSTrigger:
	sts		PPSTrigger,r16			;Store new Mode
	rcall	UpdatePPSTrigger_EEprom
	rcall	InitiateSysReset
	rjmp	clctail2



#if FUNMODES
;
; Check if any Fun Mode is active
;
; Result: r16 == 1 or 0
;
TestPlayModes:
	ldd		r16,CurrPlayMode
	cpi		r16,FUNNORM
	brne	TestFML100
	clr		r16
	rjmp	TestFML200
TestFML100:
	mov		r16,_1
TestFML200:
	ret
;
; Disable all Fun Modes
;
DisablePlayModes:
	ldd		r16,CurrPlayMode		;current mode
	sbrs	r16,RUNCLOCKFAST		;was clock running fast
	rjmp	DisableFML200			;no
	ldi		r16,TIMEOUTTICKS		;fast was on, turn off
	std		ReloadPPSTimeOutCnt,r16
	std		PPSTimeOutCnt,r16
DisableFML200:
	stdi	CurrPlayMode,FUNNORM
	CALL	SetActualTime
	ret
;
; Update Fun Mode.
;
; IN: r16 new fun mode
;
UpdPlayMode:
	ldd		r17,CurrPlayMode		;current Fun Mode
	eor		r17,r16					;change bits
	sbrs	r17,RUNCLOCKFAST		;change in Fast Mode?
	rjmp	UpdFML200				;no
;
; change in Fast Mode. Just set the new mode
;
	ldi		r17,TIMEOUTTICKS		;preset to normal
	sbrc	r16,RUNCLOCKFAST
	ldi		r17,FASTTIMEOUTTICKS	;change to Fast
	std		ReloadPPSTimeOutCnt,r17
	std		PPSTimeOutCnt,r17
UpdFML200:
	std		CurrPlayMode,r16			;set new mode
	CALL	SetActualTime
	ret
#endif		;FUNMODES
;
; Update UsrName mode
;
; IN r16 - new mode
;
UpdUsrNameMode:
#if DEBUG
	cpi		r16,USRNAMEON+1
	brsh	UpdUNML300
#endif
	sts		UsrNameFlg,r16
	rcall	UpdateUsrNameFlg	;update EEPROM
	CALL	SetDialDisplay
UpdUNML300:
	ret
;
; ToggleLedBlinkin
;
; This assumes the flag is always 0 or 1, nothing else
;
ToggleLedBlinking:
	lds		r16,LedDisabled
	eor		r16,_1					;Toggle
;
; FALL THROUGH
;
; UpdateLedBlinking
;
; IN:	r16 -- new Led Disabled Value
;
UpdateLedBlinking:
	sts		LedDisabled,r16		;update mode
	sbrc	r16,0				;skip if not disabled
	rcall	TurnOffLed
	rcall	UpdateLED_EEprom	;r16 is parameter
	rjmp	clctail2
;
; Toggle BurnIn Prevention Flag
;
; This assumes the flag is always 0 or 1, nothing else
;
ToggleBurnInPrevention:
	lds		r16,BurnInDisabled
	eor		r16,_1					;Toggle flag value
;
; FALL THROUGH
;
; UpdateBurnInPrevention
;
; IN r16 -- new flag value
;
UpdateBurnInPrevention:
	sts		BurnInDisabled,r16
UpdBurn_L50:
	push	r16					;save value of flag
	rcall	UpdateBurn_EEprom	;r16 is parameter
	pop		r16					;restore
	rcall	SetRotation			;r16 is argument
	rjmp	clctail2
;
; Toggle Verbose setting
; This assumes the flag is always 0 or 1, nothing else
;
ToggleVerbose:
	lds		r16,Verbose
	eor		r16,_1					;Toggle flag value
	sts		Verbose,r16
	rcall	UpdateVerb_EEprom		;r16 is parameter
	rjmp	clctail2
;
; Toggle Chronometer Display
;
; This assumes the flag is always 0 or 1, nothing else
;
ToggleChronoDisplay:
	lds		r16,ChronoEnabled		;current Chronovalue
	eor		r16,_1					;toggle
;
; FALL THROUGH
;
; UpdateChronoDisplay
;
; IN r16 - new Chrono value
;
UpdateChronoDisplay:
	sts		ChronoEnabled,r16
	rcall	UpdateChrono_EEprom
	CALL	SetNumericDisplay
clctail2:
	clc
	ret
;
; UpdateDSTMode
;
; IN r16 -- new mode

ChangeDSTOptions:
UpdateDSTMode:
	sts		DSTMode,r16			;must be proper NUMERIC value (0..2)
	rcall	UpdateDSTMode_EEprom
;
; check if the new mode requires DST adjustments
;
	RCALL	DSTRetroApply
	ret
;
;------------- Execute a read or write on the EEPROM (depending on R17) ---------
; 					 if R17 == 6 then Write, if R17 == 1 then Read
;
EepromTalk: 					
	out EEARL,XL 					; EEARL = address low
	out EEARH,XH 					; EEARH = address high
	adiw R26,1 						; address++
	sbrc R17,1 						; skip if R17 == 1 (read Eeprom)
#if (Mega168 | Mega328p)
	sbi EECR,EEMPE 					; EEMPE = 1 (write Eeprom)
#elif Mega32
	sbi EECR,EEMWE 					; EEMWE = 1 (write Eeprom)
#endif
	out EECR,R17 					; EECR = R17 (6 write, 1 read)
L90: 
#if (Mega168 | Mega328p)
	sbic EECR,EEPE 					; wait until EEPE == 0
#elif Mega32
	sbic EECR,EEWE 					; wait until EEWE == 0
#endif
	rjmp L90
	in R16,EEDR 					; R16 = EEDR
	ret

;-----------------------------------------------------------------------------:
; 8bit/8bit Unsigned Division
;
; Register Variables
;  Call:  r16 = dividend (0x00..0xff)
;         r17 = divisor (0x01..0x7f)
;         r18  = <don't care>
;         r18   = <don't care> (high register must be allocated)
;
;  Result:r16 = r16 / r17
;         r17 = <not changed>
;         r18  = r16 % r17
;         r19   = 0
;
; Size  = 11 words
; Clock = 66..74 cycles  (+ret)
; Stack = 0 byte
div08u:
		clr	r18		;initialize variables
		ldi	r19,8		;  mod = 0; lc = 8;
					;---- calculating loop
		lsl	r16		;var1 = var1 << 1;
		rol	r18		;mod = mod << 1 + carry;
		cp	r18,r17		;if (mod => var2) {
		brcs	PC+3		; mod -= var2; var1++;
		inc	r16		; }
		sub	r18,r17		;/
		dec	r19		;if (--lc > 0)
		brne	PC-7		; continue loop;
		ret
;
; ENHANCEMENT to V3.3
; Much faster division by 10
;
;source: http://www.avrfreaks.net/index.php?name=PNphpBB2&file=viewtopic&t=37150
; Dirty Math topic
;
; Division by 10, using reciprocal multiplication 
; 
; Call with: 
;   8 bit dividend in r16 
; 
; Returns: 
;   Result in r16 
;   Remainder in r18 
;
div08uby10:
   mov   r18,r16		; save original dividend 
   ldi   r17,26			; reciprocal, scaled *256, off a bit 
   mul   r16,r17 
;
; result of multiplication now in r1:r0 
; use only r1, thereby effectively divide by 256 
;
   dec   r1 	    	; if imprecise scaling value influences result, 
						; result will be '+1'. Decrement to avoid neg. 
						; value in later subtraction 
   ldi   r17,10			; re-use r17 for divisor 
   mov   r16,r1			; save result/256 
   mul   r16,r17		; find remainder by multiplication of result by 10d 
;
; result again in r1:r0 
; get remainder and correct result of 'by 10' 
;
   sub   r18,r0 
   cpi   r18,10 
   brlo  div08u10done 
   subi   r18,10 
   inc   r16 
div08u10done: 			; result in r16, remainder in r18 
	ret
;
; convert BCD number in r24 to binary in r24
;
bcdtobin:
	push	r16
	push	r24
	swap	r24			;get high nibble. 0..15 value
	andi	r24,0x0f
	ldi		r16,10		;times 10
	mul		r24,r16		;result in r0:r1
	mov		r16,r0
	pop		r24
	andi	r24,0x0f	;low nibble
	add		r24,r16		;result
	pop		r16
	ret
;
; convert binary number in r24 to BCD.
; Result in r16 and r24.
; uses r16,r17,r18,r19
;
bintobcd:
	mov		r16,r24		;divisor
	rcall	binto2digits
	swap	r16			;set high nibble
	or		r16,r18		;merge in low nibble
	mov		r24,r16		;set result
	ret
;
; split an 8-bit binary value into 2 decimal digits
;
; argument: r16 binary value
;
; results:	r18 low decimal digit
;			r16 high decimal digit
;
; uses:		r17,r18,r19
;
binto2digits:
;
; ENHANCEMENT to V3.3 and up
; Much faster division by 10
;
	rcall	div08uby10	;divide r16 by 10. Result in r16, Remainder in r18
	cpi		r16,10		;force range limitation (max digit is 9)
	brlt	bint2dL200
	mov		r16,_1
bint2dL200:
	ret
;
; convert nibble value in r24 into Hex Char in r24
;
Hex2Asc:
	movw	r0,ZL		;save
	ldiw	Z,ASC_TAB*2
    add     ZL,r24
    adc     ZH,_0
    lpm		r24,Z
	movw	ZL,r0		;restore
    ret

;
; Disable Watchdog Timer Completely
;
#if Mega168 | Mega328p
WDTimerOff:
	wdr
	in		r16,MCUSR
	andi	r16,~(1<<WDRF)
	out		MCUSR,r16
	lds		r16,WDTCSR
	ori		r16,(1<<WDCE)|(1<<WDE)
	sts		WDTCSR,r16
	clr		r16
	sts		WDTCSR,r16
	ret
;
; Initiate a System Reset
;
InitiateSysReset:
	cli
	wdr
	lds		r16,WDTCSR
	ori		r16,(1<<WDCE)|(1<<WDE)
	sts		WDTCSR,r16
	ldi		r16,(1<<WDE)| (0<<WDP2)|(1<<WDP1)|(0<<WDP0)	;Watchdog System Reset Enable, 64 mSec timeout
	sts		WDTCSR,r16
	sei
	rjmp	PC
	ret
#elif Mega32
WDTimerOff:
	wdr
	in		r16,MCUSR
	andi	r16,~(1<<WDRF)
	out		MCUSR,r16
	in		r16,WDTCR
	ori		r16,(1<<WDTOE)|(1<<WDE)
	out		WDTCR,r16
	clr		r16
	out		WDTCR,r16
	ret
;
; Initiate a System Reset
;
InitiateSysReset:
	cli
	wdr
	in		r16,WDTCR
	ori		r16,(1<<WDTOE)|(1<<WDE)
	out		WDTCR,r16
	ldi		r16,(1<<WDE)| (0<<WDP2)|(1<<WDP1)|(0<<WDP0)	;Watchdog System Reset Enable, 64 mSec timeout
	out		WDTCR,r16
	sei
	rjmp	PC
	ret
#endif

.include "./uart.asm"

.include "./rtc.asm"

.include "./FlashSupport.asm"

#if MULTIAPP
#include "./terminal.asm"
#endif

.include "./UsrCommand.asm"

#if DEMOMODE

;
; DemoSetUsrName
;
; Temporarily set the UsrNameBuf and UsrNameCnt for demo purposes
; Demo mode will reboot when done, restoring originals
;
; IN	Z   -- DemoName (in flash, NULL terminated)
;
; USES	r0, r16, X
;
DemoSetUsrName:
	clr		r16					;Cnt
	ldiw	X,UsrNameBuf		;Destination
DemoSUNL100:
	lpm		r0,Z+
	st		X+,r0
	inc		r16
	tst		r0					;End of String?
	brne	DemoSUNL100
	dec		r16					;correct for terminator
	sts		UsrNameCnt,r16		;Cnt
	sts		UsrNameFlg,_0		;turn on User Name Display
	CALL	SetDialDisplay		;re-center
	ret
;
; Wait 4 real second.  Use TC1 for this.
; Issue: These wait functions are all called from UsrDemo, which is called from the
;		 Menu while a TC1 timeout is running. At this point, the Menu Timeout test
;		 is never done while UsrDemo is running.
;
Special_Wait4:
	CALL	StartTC1Countdown4Sec
Special_WL100:
	CALL	SpecialClockWorks
#if GPS
	CALL	DiscardGPSInput
#endif
	CALL	TestTC1Countdown
	tst		r16
	brne	Special_WL100
; no need to stop TC1 since countdown value is now 0
	ret

.include ".\demomode.asm"
#endif

.include "./DebugSupport.asm"

;
; Help Text Data
;
; Note: Program Space, each line needs to be even length! zero padding will stop the display
;
#if EEPROMHELP
	.eseg
	.org	EEPROM_DEMOHELP
#endif		; EEPROMHELP
UsrDemoHelpText:
	.db	"This demo mode shows some of the capabilities of the",13,10
	.db	"AVR Clock. The demo preserves the state of your clock",13,10,0
#if EEPROMHELP
	.eseg
	.org	EEPROM_HELP
#endif		; EEPROMHELP

UsrHelpText:
	.db	"1234567890"
	.db	" Help:",13,10
	.db	"A: Application Selection",13,10
	.db	"B: Burn-in prevention ",13,10
	.db	"C: Calibration pattern", 13,10
	.db	"D: Change Date",13,10
	.db	"F: Clock Face Mode (0..4) ",13,10
	.db	"G: GPS Input",13,10
	.db	"I: Demo mode",13,10
	.db	"K: Chrono Display ",13,10
	.db	"L: LED blinking ",13,10
	.db	"M: PPS Trigger Mode ",13,10
	.db	"N: Numeric Display Mode (0..4)",13,10
	.db	"P: Play Mode (0..3) ",13,10
	.db	"R: Reset EEProm Config",13,10
	.db	"S: Display Status ",13,10
	.db "T: Change Time",13,10
	.db "U: Change DST mode",13,10
	.db	"V: Verbose Output ",13,10
	.db	"W: Change Name",13,10
	.db	0,0

#if EEPROMHELP
	.cseg
#endif		; EEPROMHELP

CmdMapping:
	.dw		UsrChangeApp		;'A'
	.dw		ToggleBurnInPrevention ;'B'
	.dw		ToggleCalibration	;'C'
	.dw		UsrSetDate			;'D'
	.dw		UsrDebugReport		;'E' DEBUG only
	.dw		UsrChangeDialDisplay ;'F'
	.dw		GPSToggleMode		;'G'
	.dw		UsrDisplayHelp		;'H'
	.dw		UsrDemo				;'I'
	.dw		UsrReboot			;'J'
	.dw		ToggleChronoDisplay	;'K'
	.dw		ToggleLedBlinking	;'L'
	.dw		TogglePPSTrigger	;'M'
	.dw		UsrChangeNumericDisplay ;'N'
	.dw		UsrShowCopyright	;'O'
	.dw		UsrChangePlayMode	;'P'
	.dw		UsrNoCommand		;'Q'
	.dw		UsrResetEEprom		;'R'
	.dw		UsrShow				;'S'
	.dw		UsrSetTime			;'T'
	.dw		UsrChangeDSTOptions	;'U'
	.dw		ToggleVerbose		;'V'
	.dw		UsrGetName			;'W'
	.dw		UsrCancel			;'X'
	.dw		UsrDebugSetting		;'Y' DEBUG only
	.dw		UsrChangeBeamParking ;'Z'

MsgPszCommand:			.db "Command(A,B,C,D,F,G,H,I,K,L,M,N,P,R,S,T,U,V,W):",0
MsgPszDSTApplied:		.db	"DST fix ",0,0
#if DEBUG
UART_pszOn:				.db	"On",0,0
UART_pszOff:			.db	"Off",0
#endif

MsgPszLogo:				.db	"Dutchtronix",0
MsgPszVersion:			.db	"V3"
#if MinorVersion==0
						.db	".0",0,0
#elif MinorVersion==1
						.db	".1",0,0
#elif MinorVersion==2
						.db	".2",0,0
#elif MinorVersion==3
						.db	".3",0,0
#elif MinorVersion==4
						.db	".4",0,0
#elif MinorVersion==5
						.db	".5a",0
#else
	UNSUPPORTED
#endif

MsgPszSignOn:			.db	" AVR Oscilloscope Clock ",0,0
MsgPszHourCommand:		.db	"2 digit hrs: ",0
MsgPszMinCommand:		.db	"2 digit mins:  ",0
MsgPszSecCommand:		.db	"2 digit secs:  ",0
MsgPszYearCommand:		.db	"2 digit year:    ",0
MsgPszMonthCommand:		.db	"2 digit month:   ",0
MsgPszDayCommand:		.db	"2 digit day:     ",0
MsgPszCancel:			.db	" Cancel/Timeout",13,10,0
MsgPszNumericCommand:	.db	"2 digit value ",0,0
MsgPszNumfieldValues:
MsgPszDialfieldValues:	.db	"(0-4):",0,0
MsgPszDSTfieldValues:	.db	"(0-2):",0,0
#if MULTIAPP
MsgPszChangeAppValues:
#endif
MsgPszFunfieldValues:	.db	"(0-3):",0,0
MsgPszGetName:			.db "Name ",0
MsgPszCopyright:		.db	"(C)2008 Jan de Rie"		;continued on next line
MsgPszNewLine:			.db	CR,LF,0,0


#if MULTIAPP
#include "./minidds.asm"
#endif
