;
;Description
;
; Poor-mans DDS Synthesizer
; 
; Author = Jesper Hansen
; Ported to Dutchtronix AVR Oscilloscope Clock by Jan de Rie
; Target = ATmega168/Atmega328p
; Date   = 2001-02-15, June 27,2007, October 30, 2008
;
;
; see dac.h for DAC pins
; see uart.h for Serial pins
;
; r25, r28, r29 not available (G_FLAGS and dedicated Y register)
;
;*******************************************************************
;*******************************************************************
;
;
;
; Output frequency (using 24 bit accumulator) :
;
;	f = deltaPhase * fClock/2^24
;
;   fClock is in this case the CPU clock divided by the
;	number of cycles to output the data (13 cycles ->1538461.5)
;
;	f = r22/r23/r24 * (1538461.5/16777216)
;
;	f = r22/r23/r24 * 0.0916994
;
;	fMax (theoretical) = 0.5 * fClock
;	Observed decent frequency: 200 KHz (adder value 0x002147AF)
;
#define ASCIILINK 0

;******************************************************************************
; code
;******************************************************************************
StartMiniDDS:
	sbi		GPIOR0,fMiniDDS		;Set MiniDDS mode
	cli
	call	DisableINT0			;Stop INT0 interrupts
	call	DisableTC0			;Stop TC0 Timer Interrupts
	call	DisableTC1			;Stop TC1 Timer Interrupts	Check may not be running
	call	DisableTC2			;Stop TC2 Timer Interrupts
#if !ASCIILINK
	call	UART_Set115200		;DDSControl.exe expects 115,200
#endif
	sei
	ret

#if 1
;
; Stopping MiniDDS is not trivial since we don't want to poll
;
; Suggested solution: From the serial input function, when an exit
;	command is detected, PATCH the code in wavegen to stop the loop
;	and execute StopMiniDDS
;

StopMiniDDS:
	cli
;	rcall	UnpatchCode			;undo the code patch
;	set stack properly
	call	EnableTC2			;Start TC2 Timer Interrupt
	call	EnableTC1			;Start TC1 Timer Interrupt	
	call	EnableTC0			;Start TC0 Timer Interrupts
	call	EnableINT0			;Start INT0 interrupts
#if !ASCIILINK
	call	UART_Set19200		;Clock expect 19,200
#endif
	cbi		GPIOR0,fMiniDDS		;Clr MiniDDS mode
;
; no option of testing for exit in wavegen loop.
; could patch code to some exit function which
; should unpatch the code and return to the clock
;
	call	InitiateSysReset
	sei	
	ret
#endif

RunFuncGen:
RunMiniDDS:
	rcall	StartMiniDDS		

; set sinewave output as default
		
	ldiw	Z,sine*2		; setup Z pointer. Always at 0x100 boundary
#if DEBUG
	tst		ZL
	breq	RunMDDSL100
	rjmp	PC
RunMDDSL100:
#endif

; clear accumulator 

	ldi 	r21,0x00		; clear accumulator 
	ldi 	r20,0x00		; clear accumulator 

; setup adder registers		
	
	ldi 	r22,0x9A		; setup adder value
	ldi 	r23,0x2A		; to 1 kHz
	ldi 	r24,0x00		; 

;
; initialize DAC to always use DACA
;
	cbi		DACSELECT			;select DACA (active low)
	cbi		DACLDAC				;update DAC channels continuously
	BEAMON
;
; wavegen loop
;
;	r20,r21,r30 is the phase accumulator
;  	r22,r23,r24 is the adder value determining frequency
;
; 	add value to accumulator
;	load byte from current table in ROM
;	output byte to DAC
;	repeat 
;
wavegen:
	add		r20,r22			;1
	adc		r21,r23			;1
	adc		r30,r24			;1
	lpm						;3
	out		DACLODATA,r0	;1 upper 4 pins are set as inputs
	swap	r0				;1
	out		DACHIDATA,r0	;1 upper 4 pins are set as inputs
	cbi		DACWR			;1 latch data. necessary for AD7302
	sbi		DACWR			;1
	rjmp	wavegen			;2 => 13 cycles


;**********************************************************************
; communication functionality
;**********************************************************************
;
; get char in r16
;
get_byte:
	lds		r16,UCSR0A		; wait for a byte to be ready
	sbrs	r16,RXC0		; ready ?
	rjmp	get_byte		; no, wait some more
	lds		r16,UDR0		; get the byte
	ret						; and return
;
; send char in r16
;
send_byte:
	push	r16				; save r16
send_c2:
	lds		r16,UCSR0A		; wait for the transmitter to be ready
	sbrs	r16,UDRE0		; ready ? 
	rjmp	send_c2			; no, wait some more
	pop		r16				; restore r16
	sts		UDR0,r16		; send char
	ret						; and return

wait_10ms:
	clr		r16
wait_ms_0:
	clr		r17
wait_ms_1:
	subi	r17,-1			; 1
	brne	wait_ms_1		; 2 -> 3 x 256=768
	subi	r16,-1			; 1
	brne	wait_ms_0		; 2 -> 771 * 256 = 197376
	ret
;	
; send the current adder value (frequency) to the PC
; as a 5 byte sequence :
; 'F' folowed by a 32 bit phase accumulator value
;	  or 4 hex values in ASCIILINK
;
send_data:
	push	r16
	push	r17
	push	r18
	ldi		r18,1
send_data_1:
	rcall	wait_10ms
	subi	r18,1
	brne	send_data_1
	pop		r18
	pop		r17
	pop		r16

		ldi		r16,'F'			; flag
		call	send_byte
#if ASCIILINK
		clr		r16				; zero byte for 32-bit compatibility
		call	Printfu08		; MSB

		mov		r16,r24
		call	Printfu08		; high add

		mov		r16,r23
		call	Printfu08		; mid add

		mov		r16,r22
		call	Printfu08		; low add

		ldi		r16,0x0d
		call	send_byte		; terminator
		ldi		r16,0x0a
		call	send_byte		; terminator
#else
		clr		r16				; zero byte for 32-bit compatibility
		CALL	send_byte		; MSB

		mov		r16,r24
		CALL	send_byte		; high add

		mov		r16,r23
		CALL	send_byte		; mid add

		mov		r16,r22
		CALL	send_byte		; low add

		ldi		r16,0x0a
		CALL	send_byte		; terminator
#endif
		ret
;
; Semantics reinterpretation: change FREQUENCY, not ACCUMULATOR
;
; Resolution is 0.0916994 => multiplier is 10.905
;
; add 1 to the frequency => add 10.9 to the accumulator
up_one:
		ldi		r17,11
		add		r22,r17
		adc		r23,_0
		adc		r24,_0
;
; TODO: now subtract 0.1 from the accumulator
		ret
;
; add 10 to the frequency => add 109 to the accumulator
; missing: 0.05
;
up_ten:
		ldi		r17,109
		add		r22,r17
		adc		r23,_0
		adc		r24,_0
		ret
;
; add 100 to the frequency => call up_ten 10 times
; missing: 0.5
;
up_hundred:
		ldi		r18, 10
up_hundred_1:
		rcall	up_ten
		dec		r18
		brne	up_hundred_1
		ret
;
; add 10000 to the frequency
;
up_tenthousand:
		ldi		r16,100
up_h10:
		rcall	up_hundred
		dec		r16
		brne	up_h10
;
; now add another 50 for the fraction
;
		ldi		r17,50
		add		r22,r17
		adc		r23,_0
		adc		r24,_0
		ret
;
; subtract 1 from the frequency => sub 10.905 to the accumulator
;
down_one:
		subi	r22,11
		sbc		r23,_0
		sbc		r24,_0
;
; TODO: now add 0.1 to the accumulator
		ret
;
; subtract 10 from the frequency => sub 109.05 to the accumulator
;
down_ten:
		subi	r22,109
		sbc		r23,_0
		sbc		r24,_0
		ret
;
; subtract 100 from the frequency => call down_ten 10 times
;
down_hundred:
		ldi		r18, 10
down_hundred_1:
		rcall	down_ten
		dec		r18
		brne	down_hundred_1
		ret

; subtract 10000 from the frequency
down_tenthousand:
		ldi		r16,100
down_h10:
		rcall	down_hundred
		dec		r16
		brne	down_h10
;
; now subtract another 50 for the fraction
;
		subi	r22,50
		sbc		r23,_0
		sbc		r24,_0
		ret

#if ASCIILINK
;
; print byte in r16 as hex value
;
Printfu08:
	push	r24
	push    r16
    swap    r16
    andi    r16,0X0F
	mov		r24,r16
    call	Hex2Asc
	mov		r16,r24
	rcall	send_byte
    pop     r16
    andi    r16,0X0F
	mov		r24,r16
    call 	Hex2Asc
	mov		r16,r24
	rcall	send_byte
	pop		r24
	ret
;
; read 6 hex characters (24-bit value) from the serial link
; no error checking
;
readAdderValue:
	rcall	getnibble		;bits 20..23
	mov		r17,r16			;save
	swap	r17
	rcall	getnibble		;bits 16..19
	or		r17,r16			;merge
	mov		r26,r17			;MSB
	rcall	getnibble		;bits 12..15
	mov		r17,r16			;save
	swap	r17
	rcall	getnibble		;bits 8..11
	or		r17,r16			;merge
	mov		r25,r17			;MIDDLE SB
	rcall	getnibble		;bits 4..7
	mov		r17,r16			;save
	swap	r17
	rcall	getnibble		;bits 0..3
	or		r17,r16			;merge
	mov		r24,r17			;LSB
	ret
	
getnibble:
	rcall	get_byte
	subi	r16,'0'			;map to binary
	brcs	error			;too small
	cpi		r16,9			;lower range?
	brcs	done
	subi	r16,7			;upper range
done:
	ret
error:
	clr		r16
	ret

#else
; 
; read in 4 characters from the serial link
;
readAdderValue:
		rcall	get_byte		; read and ignore bits 32..24
		rcall	get_byte		; read bits 23..16
		mov		r24,r16
		rcall	get_byte		; read bits 15..8
		mov		r23,r16
		rcall	get_byte		; read bits 7..0
		mov		r22,r16
		ret
#endif
;
; Interrupt routine for incoming bytes on the RS232 link
;
RX_COMPLETE_INT:
;
; Check if the push button switch S2 is closed.
; Reboot if it is
;
		sbis	SW_PIN,SW2_BIT			;skip if HIGH (meaning UP)
		CALL	InitiateSysReset

		push	r17
		push	r18
		lds		r16,UDR0
		call	send_byte
		cpi		r16,'-'				; up 1
		brne	tx_2
		rcall	up_one
		rjmp	tx_exit		
tx_2:
		cpi		r16,'d'				; up 100
		brne	tx_3
		rcall	up_hundred
		rjmp	tx_exit		
tx_3:
		cpi		r16,'D'				; up 10,000
		brne	tx_4
		rcall	up_tenthousand
		rjmp	tx_exit		
tx_4:
		cpi		r16,'+'				; down 1
		brne	tx_5
		rcall	down_one
		rjmp	tx_exit		
tx_5:
		cpi		r16,'u'				; down 100
		brne	tx_6
		rcall	down_hundred
		rjmp	tx_exit		
tx_6:
		cpi		r16,'U'				; down 10,000
		brne	tx_7
		rcall	down_tenthousand		
		rjmp	tx_exit		
tx_7:
		cpi		r16,'s'				; frequency setting
		brne	tx_8
		rcall	readAdderValue		
		rjmp	tx_exit		
tx_8:
		cpi		r16,'?'				; just force a reply
		brne	tx_9
		rjmp	tx_exit		
tx_9:	
		cpi		r16,'1'				; request sinewave output
		brne	tx_10
		ldiw	Z,sine*2			; setup Z pointer to sine

		rjmp	tx_exit
tx_10:
		cpi		r16,'2'				; request sawtooth output
		brne	tx_11
		ldiw	Z,sawtooth*2		; setup Z pointer to sawtooth
		rjmp	tx_exit
tx_11:
		cpi		r16,'3'				; request triangle output
		brne	tx_12
		ldiw	Z,triangle*2		; setup Z pointer to triangle
		rjmp	tx_exit
tx_12:
		cpi		r16,'4'				; request squarewave output
		brne	tx_13
		ldiw	Z,square*2			; setup Z pointer to square
		rjmp	tx_exit
tx_13:
		cpi		r16,'q'				;exit MiniDDS
		brne	tx_14
		rcall	StopMiniDDS
; unknown command, just ignore it
tx_14:

; always reply with the current adder value
tx_exit:
		rcall	send_data
		pop		r18
		pop		r17
;
; exit code has to match SIG_USART_RECV in uart.asm
;
		pop		r16
		out		SREG,r16
		pop		r16
		reti

;
; Data tables for supported wave forms. 1K
; These are located at the end of the flash, just before the bootloader.
; Force table to be aligned on a 256 byte boundary, so that
; we never have to update register ZH
;
#if Mega168			// 1KB bootloader
	.org 0x1c00
#elif Mega328p		// 1KB bootloader
	.org 0x3c00
#elif Mega32		// no bootloader
	.org 0x3c00
#endif

sine:		; 256 step sinewave table
	.db	0x80,0x83,0x86,0x89,0x8c,0x8f,0x92,0x95,0x98,0x9c,0x9f,0xa2,0xa5,0xa8,0xab,0xae
	.db	0xb0,0xb3,0xb6,0xb9,0xbc,0xbf,0xc1,0xc4,0xc7,0xc9,0xcc,0xce,0xd1,0xd3,0xd5,0xd8
	.db	0xda,0xdc,0xde,0xe0,0xe2,0xe4,0xe6,0xe8,0xea,0xec,0xed,0xef,0xf0,0xf2,0xf3,0xf5
	.db	0xf6,0xf7,0xf8,0xf9,0xfa,0xfb,0xfc,0xfc,0xfd,0xfe,0xfe,0xff,0xff,0xff,0xff,0xff
	.db	0xff,0xff,0xff,0xff,0xff,0xff,0xfe,0xfe,0xfd,0xfc,0xfc,0xfb,0xfa,0xf9,0xf8,0xf7
	.db	0xf6,0xf5,0xf3,0xf2,0xf0,0xef,0xed,0xec,0xea,0xe8,0xe6,0xe4,0xe2,0xe0,0xde,0xdc
	.db	0xda,0xd8,0xd5,0xd3,0xd1,0xce,0xcc,0xc9,0xc7,0xc4,0xc1,0xbf,0xbc,0xb9,0xb6,0xb3
	.db	0xb0,0xae,0xab,0xa8,0xa5,0xa2,0x9f,0x9c,0x98,0x95,0x92,0x8f,0x8c,0x89,0x86,0x83
	.db	0x80,0x7c,0x79,0x76,0x73,0x70,0x6d,0x6a,0x67,0x63,0x60,0x5d,0x5a,0x57,0x54,0x51
	.db	0x4f,0x4c,0x49,0x46,0x43,0x40,0x3e,0x3b,0x38,0x36,0x33,0x31,0x2e,0x2c,0x2a,0x27
	.db	0x25,0x23,0x21,0x1f,0x1d,0x1b,0x19,0x17,0x15,0x13,0x12,0x10,0x0f,0x0d,0x0c,0x0a
	.db	0x09,0x08,0x07,0x06,0x05,0x04,0x03,0x03,0x02,0x01,0x01,0x00,0x00,0x00,0x00,0x00
	.db	0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x01,0x02,0x03,0x03,0x04,0x05,0x06,0x07,0x08
	.db	0x09,0x0a,0x0c,0x0d,0x0f,0x10,0x12,0x13,0x15,0x17,0x19,0x1b,0x1d,0x1f,0x21,0x23
	.db	0x25,0x27,0x2a,0x2c,0x2e,0x31,0x33,0x36,0x38,0x3b,0x3e,0x40,0x43,0x46,0x49,0x4c
	.db	0x4f,0x51,0x54,0x57,0x5a,0x5d,0x60,0x63,0x67,0x6a,0x6d,0x70,0x73,0x76,0x79,0x7c

sawtooth:	; 256 step sawtoothwave table
	.db	0x00,0x01,0x02,0x03,0x04,0x05,0x06,0x07,0x08,0x09,0x0a,0x0b,0x0c,0x0d,0x0e,0x0f
	.db	0x10,0x11,0x12,0x13,0x14,0x15,0x16,0x17,0x18,0x19,0x1a,0x1b,0x1c,0x1d,0x1e,0x1f
	.db	0x20,0x21,0x22,0x23,0x24,0x25,0x26,0x27,0x28,0x29,0x2a,0x2b,0x2c,0x2d,0x2e,0x2f
	.db	0x30,0x31,0x32,0x33,0x34,0x35,0x36,0x37,0x38,0x39,0x3a,0x3b,0x3c,0x3d,0x3e,0x3f
	.db	0x40,0x41,0x42,0x43,0x44,0x45,0x46,0x47,0x48,0x49,0x4a,0x4b,0x4c,0x4d,0x4e,0x4f
	.db	0x50,0x51,0x52,0x53,0x54,0x55,0x56,0x57,0x58,0x59,0x5a,0x5b,0x5c,0x5d,0x5e,0x5f
	.db	0x60,0x61,0x62,0x63,0x64,0x65,0x66,0x67,0x68,0x69,0x6a,0x6b,0x6c,0x6d,0x6e,0x6f
	.db	0x70,0x71,0x72,0x73,0x74,0x75,0x76,0x77,0x78,0x79,0x7a,0x7b,0x7c,0x7d,0x7e,0x7f
	.db	0x80,0x81,0x82,0x83,0x84,0x85,0x86,0x87,0x88,0x89,0x8a,0x8b,0x8c,0x8d,0x8e,0x8f
	.db	0x90,0x91,0x92,0x93,0x94,0x95,0x96,0x97,0x98,0x99,0x9a,0x9b,0x9c,0x9d,0x9e,0x9f
	.db	0xa0,0xa1,0xa2,0xa3,0xa4,0xa5,0xa6,0xa7,0xa8,0xa9,0xaa,0xab,0xac,0xad,0xae,0xaf
	.db	0xb0,0xb1,0xb2,0xb3,0xb4,0xb5,0xb6,0xb7,0xb8,0xb9,0xba,0xbb,0xbc,0xbd,0xbe,0xbf
	.db	0xc0,0xc1,0xc2,0xc3,0xc4,0xc5,0xc6,0xc7,0xc8,0xc9,0xca,0xcb,0xcc,0xcd,0xce,0xcf
	.db	0xd0,0xd1,0xd2,0xd3,0xd4,0xd5,0xd6,0xd7,0xd8,0xd9,0xda,0xdb,0xdc,0xdd,0xde,0xdf
	.db	0xe0,0xe1,0xe2,0xe3,0xe4,0xe5,0xe6,0xe7,0xe8,0xe9,0xea,0xeb,0xec,0xed,0xee,0xef
	.db	0xf0,0xf1,0xf2,0xf3,0xf4,0xf5,0xf6,0xf7,0xf8,0xf9,0xfa,0xfb,0xfc,0xfd,0xfe,0xff

triangle:	; 256 step trianglewave table
	.db	0x00,0x02,0x04,0x06,0x08,0x0a,0x0c,0x0e,0x10,0x12,0x14,0x16,0x18,0x1a,0x1c,0x1e
	.db	0x20,0x22,0x24,0x26,0x28,0x2a,0x2c,0x2e,0x30,0x32,0x34,0x36,0x38,0x3a,0x3c,0x3e
	.db	0x40,0x42,0x44,0x46,0x48,0x4a,0x4c,0x4e,0x50,0x52,0x54,0x56,0x58,0x5a,0x5c,0x5e
	.db	0x60,0x62,0x64,0x66,0x68,0x6a,0x6c,0x6e,0x70,0x72,0x74,0x76,0x78,0x7a,0x7c,0x7e
	.db	0x80,0x82,0x84,0x86,0x88,0x8a,0x8c,0x8e,0x90,0x92,0x94,0x96,0x98,0x9a,0x9c,0x9e
	.db	0xa0,0xa2,0xa4,0xa6,0xa8,0xaa,0xac,0xae,0xb0,0xb2,0xb4,0xb6,0xb8,0xba,0xbc,0xbe
	.db	0xc0,0xc2,0xc4,0xc6,0xc8,0xca,0xcc,0xce,0xd0,0xd2,0xd4,0xd6,0xd8,0xda,0xdc,0xde
	.db	0xe0,0xe2,0xe4,0xe6,0xe8,0xea,0xec,0xee,0xf0,0xf2,0xf4,0xf6,0xf8,0xfa,0xfc,0xfe
	.db	0xff,0xfd,0xfb,0xf9,0xf7,0xf5,0xf3,0xf1,0xef,0xef,0xeb,0xe9,0xe7,0xe5,0xe3,0xe1
	.db	0xdf,0xdd,0xdb,0xd9,0xd7,0xd5,0xd3,0xd1,0xcf,0xcf,0xcb,0xc9,0xc7,0xc5,0xc3,0xc1
	.db	0xbf,0xbd,0xbb,0xb9,0xb7,0xb5,0xb3,0xb1,0xaf,0xaf,0xab,0xa9,0xa7,0xa5,0xa3,0xa1
	.db	0x9f,0x9d,0x9b,0x99,0x97,0x95,0x93,0x91,0x8f,0x8f,0x8b,0x89,0x87,0x85,0x83,0x81
	.db	0x7f,0x7d,0x7b,0x79,0x77,0x75,0x73,0x71,0x6f,0x6f,0x6b,0x69,0x67,0x65,0x63,0x61
	.db	0x5f,0x5d,0x5b,0x59,0x57,0x55,0x53,0x51,0x4f,0x4f,0x4b,0x49,0x47,0x45,0x43,0x41
	.db	0x3f,0x3d,0x3b,0x39,0x37,0x35,0x33,0x31,0x2f,0x2f,0x2b,0x29,0x27,0x25,0x23,0x21
	.db	0x1f,0x1d,0x1b,0x19,0x17,0x15,0x13,0x11,0x0f,0x0f,0x0b,0x09,0x07,0x05,0x03,0x01



square:		; 256 step squarewave table
	.db	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
	.db	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
	.db	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
	.db	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
	.db	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
	.db	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
	.db	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
	.db	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
#if AVRDUDEHACK
;
; Update Oct 30, 2008.
; avrdude has a bug where it doesn't write a page (128 bytes) of data in the flash file.
; if the data are all 0xff
;
	.db	0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe
	.db	0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe
	.db	0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe
	.db	0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe
	.db	0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe
	.db	0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe
	.db	0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe
	.db	0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe,0xfe
#else
	.db	0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff
	.db	0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff
	.db	0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff
	.db	0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff
	.db	0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff
	.db	0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff
	.db	0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff
	.db	0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff
#endif
